@@ -1,6 +1,6 @@
 # -*-mode: tcl; fill-column: 75; tab-width: 8; coding: iso-latin-1-unix -*-
 #
-#       $Id: Plot3d.tcl,v 1.6 2002/09/14 17:25:35 mikeclarkson Exp $
+#       $Id: Plot3d.tcl,v 1.7 2004/03/28 07:28:27 vvzhy Exp $
 #
 ###### Plot3d.tcl ######
 ############################################################
@@ -677,7 +677,7 @@ proc mkPlot3d { win  args } {
     button $wb.rotate -text "Rotate" -command "setForRotate $win" -font $buttonFont
     setBalloonhelp $win $wb.rotate {Dragging the mouse with the left button depressed will cause the object to rotate.  The rotation keeps the z axis displayed in an upright position (ie parallel to the sides of the screen), but changes the viewpoint.   Moving right and left changes the azimuth (rotation about the z axis), and up and down changes the elevation (inclination of z axis).   The red,blue and green sides of the bounding box are parallel to the X, Y and Z axes, and are on the smaller side.}
 
-    $win.position config -width 15
+    #$win.position config -width 15
     pack $wb.rotate -expand 1 -fill x
     setForRotate $win
 
@@ -740,7 +740,7 @@ proc showPosition3d { win x y } {
 	set ll [lindex $mesh $at]
 	set pt [lrange [oget $win points] $ll [expr {$ll + 2}]]
 	# puts pt=$pt
-	catch { oset $win position [eval [concat "format {(%.2f %.2f %.2f)}" $pt]] }
+	catch { $win.plotmenu config -text [eval [concat "format {(%.2f %.2f %.2f)}" $pt]] }	
     }
     #    oset $win position [format {(%.1f %.1f)} $x $y]
     #    oset $win position \
