@@ -58,10 +58,10 @@
 	     (add2lnc tensor $symmetries)
 	     (return '$DONE)))
 
-(defun INTERVAL (i j)     ;INTERVAL returns the list of integers from I thru J.
-       (do ((n i (1+ n)) (ans))             ;Thus (INTERVAL 3 5) yields (3 4 5)
-           ((> n j) (nreverse ans))
-           (setq ans (cons n ans))))
+;(defun INTERVAL (i j)     ;INTERVAL returns the list of integers from I thru J.
+;       (do ((n i (1+ n)) (ans))             ;Thus (INTERVAL 3 5) yields (3 4 5)
+;           ((> n j) (nreverse ans))
+;           (setq ans (cons n ans))))
 
 (defun CHECK-SYMARGS (ll n)            ;Returns an ascending list of the unique
 				       ;elements of LL and checks that they are
@@ -124,7 +124,7 @@
 (defun CANTEN (e nfprpobjs)                                   ;CANonical TENsor
        (prog (cov contr deriv tensor)
 	     ((lambda (dummy) (and nfprpobjs dummy (setq e (rename1 e dummy))))
-	      (cdaddr ($indices2 e)))            ;NFPRPOBJS is Not From Product
+	      (NONUMBER (cdaddr ($indices2 e)))) ;NFPRPOBJS is Not From Product
 	     (setq cov (copy (cdadr e))          ;of RP (indexed) OBJects
 		   contr (copy (cdaddr e))
 		   deriv (copy (cdddr e))
@@ -134,26 +134,28 @@
 		   csign nil)       ;Set when reordering antisymmetric indices.
                                     ;Indicates whether overall sign of
                                     ;expression needs changing.
-	     (cond ($allsym (setq cov (mysort cov) contr (mysort contr)))
-		   ((zl-member tensor (cdr $symmetries))
+	     (cond
+		   ((OR (OR (eq (caar e) '$LC) (eq (caar e) '%LC)) (OR (eq (caar e) '$KDELTA) (eq (caar e) '%KDELTA))) (setq cov (ANTISORT cov) contr (ANTISORT contr)))
+		   ((OR $allsym (eq (caar e) '$KDELS) (eq (caar e) '%KDELS)) (setq cov (itensor-sort cov) contr (itensor-sort contr)))
+		   ((zl-member ($VERBIFY tensor) (cdr $symmetries))
 		    (do ((q symtypes (cdr q)) (type))
 			((null q))
 			(setq type (car q))
-			(do ((props (car (zl-get tensor type)) (cdr props)) (p))
+			(do ((props (car (zl-get ($VERBIFY tensor) type)) (cdr props)) (p))
 			    ((null props))
 			    (setq p (car props)
-				  cov (inserts (symsort (extract p cov) type)
+				  cov (inserts (symsort (extract-elements p cov) type)
 					       cov p)))
-			(do ((props (cdr (zl-get tensor type)) (cdr props)) (p))
+			(do ((props (cdr (zl-get ($VERBIFY tensor) type)) (cdr props)) (p))
 			    ((null props))
 			    (setq p (car props)
-				    contr (inserts (symsort (extract p contr)
+				    contr (inserts (symsort (extract-elements p contr)
 							    type)
 						   contr p))))))
 	     (setq tensor (mysubst0 (append (list (car e)
 						  (consmlist cov)
 						  (consmlist contr))
-					    (mysort deriv)) e))
+					    (itensor-sort deriv)) e))
 	     (cond (csign (setq tensor (neg tensor))))
 	     (return tensor)))
 
@@ -167,8 +169,8 @@
 	   (cond ((not (eq dumx (car b)))
 		  (setq l (cons (cons (car b) dumx) l))))))
 
-(defun EXTRACT (a b)  ;Extracts the elements from B specified by the indices in
-                      ;i.e. (EXTRACT '(2 5) '(A B C D E F)) yields (B E)
+(defun EXTRACT-elements (a b)  ;Extracts the elements from B specified by the indices in
+                      ;i.e. (EXTRACT-elements '(2 5) '(A B C D E F)) yields (B E)
        (do ((a a) (b b (cdr b)) (n 1 (1+ n)) (l))
 	   ((null a) (nreverse l))
 	   (cond ((equal (car a) n)
@@ -238,14 +240,14 @@
 			      (mapcar (function (lambda (z) (canten z nil)))
 				      ((lambda (q)
 					       (rename1 q
-						       (cdaddr
+						       (NONUMBER (cdaddr
 						        ($indices2
-						         (cons '(MTIMES) q)))))
+						         (cons '(MTIMES) q))))))
 				       (mapcar 'cdr
 					       (sortcar
 						(progn
 						 (setq free-indices
-						       (cdadr ($indices2 e)))
+						       (NONUMBER (cdadr ($indices2 e))))
 						 (mapcar 'describe-tensor
 							 indexed))
 						'tensorpred))))))))))
