@@ -43,7 +43,7 @@
 		(APPLY F L))
     #+cl
     ((and (symbolp f)(or (macro-function f)
-			 (special-form-p f)))
+			 (special-operator-p f)))
      (eval (cons f (copy-rest-arg l))))
     (T
      (MAPPLY F (copy-rest-arg  L) NIL))))
@@ -60,6 +60,7 @@
      (CASE (ARRAY-TYPE AARRAY)
 	    ((FLONUM FIXNUM #+LISPM ART-Q #+cl t)
 	     (apply 'aref AARRAY IND1 INDS))
+	    #-cl
 	    ((T)
 	     (MARRAYREF-GENSUB AARRAY IND1 INDS))
 	    (T
@@ -143,6 +144,7 @@
 	#-cl (STORE (APPLY AARRAY IND1 INDS) VAL)
 	#+cl (setf (apply #'aref aarray ind1 inds) val)
 	)
+       #-cl
        ((T)
 	(MARRAYSET-GENSUB VAL AARRAY IND1 INDS))
        (T
