@@ -326,14 +326,14 @@
 (comment euler and bernoulli stuff)
 
 #+cl
-(progn
-(defvar *bn* #(nil -1. 1. -1. 5. -691. 7. -3617. 43867. -174611. 854513.
+(eval-when (compile eval load)
+(defvar *bn* (vector nil -1. 1. -1. 5. -691. 7. -3617. 43867. -174611. 854513.
 		   -236364091.
 		8553103. -23749461029. 8615841276005. -7709321041217.
 		2577687858367.))
-(defvar *bd* #(nil 30. 42. 30. 66. 2730. 6. 510. 798. 330. 138. 2730. 6. 870. 14322.
+(defvar *bd* (vector nil 30. 42. 30. 66. 2730. 6. 510. 798. 330. 138. 2730. 6. 870. 14322.
         510. 6.))
-(defvar *eu* #(-1. 5. -61. 1385. -50521. 2702765. -199360981. 19391512145.
+(defvar *eu* (vector -1. 5. -61. 1385. -50521. 2702765. -199360981. 19391512145.
 	    -2404879675441. 370371188237525. -69348874393137901.))
 )
 
@@ -351,7 +351,7 @@
 
 
 (setq i 0)
-(mapc '(lambda (a)
+(mapc #'(lambda (a)
 	   (store (aref *bd* (setq i (f1+ i))) a))
       '(30. 42. 30. 66. 2730. 6. 510. 798. 330. 138. 2730. 6. 870. 14322.
         510. 6.))
@@ -1080,7 +1080,7 @@
 			   (null
 			    (or (numberp (setq d (simplus (list '(mplus) h (list '(mtimes) -1 n)) 1 nil)))
 				(and (numberp (simplus (list '(mplus) n (list '(mtimes) -2 h)) 1 nil))
-				     (setq fl 2)))))
+				     (progn (setq fl 2) t)))))
 		       (return (adusum e))))
 		(setq e (list '(%binomial) n *var*))
 		(and (greaterp l 0) (adsum (dosum (list '(mtimes) y -1 e) *var* 0 (sub1 l) t)))
