@@ -596,6 +596,13 @@
 		    (REVERSE (CDR E))
 		    (CDR E))))))
 
+;;; These functions implement the Macsyma functions $op and $operatorp.
+;;; Dan Stanger
+(defmfun $op (expr) ($part expr 0))
+
+(defmfun $operatorp (expr oplist)
+   (if ($listp oplist) ($member ($op expr) oplist) (equal ($op expr) oplist)))
+
 (DEFMFUN $PART N (MPART (LISTIFY N) NIL NIL $INFLAG '$PART))
 
 (DEFMFUN $INPART N (MPART (LISTIFY N) NIL NIL T '$INPART))
@@ -630,7 +637,7 @@
 		   ((SPECREPP EXP) (SETQ EXP (SPECDISREP EXP))))
 	     (IF (AND (ATOM EXP) (NULL $PARTSWITCH))
 		 (MERROR "~:M called on atom: ~:M" FN EXP))
-	     (IF (AND INFLAG SPECP) (SETQ EXP (copy-all-levels EXP)))
+	     (IF (AND INFLAG SPECP) (SETQ EXP (copy-tree EXP)))
 	     (SETQ EXP* EXP)
 	START(COND ((OR (ATOM EXP) (EQ (CAAR EXP) 'BIGFLOAT)) (GO ERR))
 		   ((EQUAL (SETQ ARG (COND (SUBSTFLAG (MEVAL (CAR ARGLIST)))
@@ -666,7 +673,7 @@
 							   '(ARRAY))))
 				     (RESIMPLIFY EXP*))
 				    (DISPFLAG
-				     (RPLACD EXP (CDR (BOX (copy-all-levels EXP) DISPFLAG)))
+				     (RPLACD EXP (CDR (BOX (copy-tree EXP) DISPFLAG)))
 				     (RPLACA EXP (IF (EQ DISPFLAG T)
 						     '(MBOX)
 						     '(MLABOX)))
@@ -821,6 +828,27 @@
   (IF (NULL (CDR E)) (MERROR "Argument to FIRST is empty."))
   (CAR (MARGS E)))
 
+; This macro is used to create functions second thru tenth.
+; Rather than try to modify mformat for ~:R, use the quoted symbol
+
+(defmacro make-nth (si i)
+ (let ((sim (intern (concatenate 'string "$" (symbol-name si)))))
+  `(DEFMFUN ,sim (E)
+     (ATOMCHK (SETQ E (FORMAT1 E)) ',sim NIL)
+     (IF (< (LENGTH (MARGS E)) ,i)
+         (MERROR "There is no ~A element:~%~M" ',si E))
+     (,si (MARGS E)))))
+
+(make-nth SECOND  2)
+(make-nth THIRD   3)
+(make-nth FOURTH  4)
+(make-nth FIFTH   5)
+(make-nth SIXTH   6)
+(make-nth SEVENTH 7)
+(make-nth EIGHTH  8)
+(make-nth NINTH   9)
+(make-nth TENTH  10)
+
 (DEFMFUN $REST N
   (PROG (M FUN FUN1 REVP)
 	(IF (AND (= N 2) (EQUAL (ARG 2) 0)) (RETURN (ARG 1)))
