@@ -122,6 +122,10 @@
 			(t `(eql ,typ ',(car all)))))
     (pushnew  (funcall *primitive-data-type-function* (eval v)) all)))
 
+;; This assumes way too much about how typep works, so don't use it.
+;; We leave it here for reference to make sure the replacement below
+;; does the same thing.
+#+nil
 (defun maclisp-typep (x &optional type)
   (cond (type
 	 (lisp:let (( pred (get type 'ml-typep)))
@@ -155,7 +159,28 @@
 	   ;((one-of-types .type. (make-array '(2 3)))  'array)
 	   (t (type-of x)))))))
 
- 
+;; A more portable implementation of maclisp-typep.  I (rtoy) think it
+;; would probably be better to replace uses of maclisp-typep and/or
+;; ml-typep with the corresponding Common Lisp typep or type-of or
+;; subtypep, as appropriate.
+(defun maclisp-typep (x &optional type)
+  (cond (type
+	 (lisp:let ((pred (get type 'ml-typep)))
+	   (cond (pred
+		  (funcall pred x))
+		 (t (typep x type)))))
+	(t
+	 (typecase x
+	   (cl:cons 'list)
+	   (cl:fixnum 'fixnum)
+	   (cl:integer 'bignum)
+	   (cl:float 'flonum)
+	   (cl:number 'number)
+	   (cl:array 'array)
+	   (cl:hash-table 'hash-table)
+	   (t
+	    (type-of x))))))
+
 (deff ml-typep #'maclisp-typep)
 ;;so that (ml-typep a 'list) ==> (zl-listp a)
 
@@ -179,6 +204,12 @@ compiler:
 (defprop :extended-number extended-number-p ml-typep)
 (defprop array arrayp ml-typep)
 (defprop atom  atom ml-typep)
+
+#+cmu (shadow '(lisp::compiled-function-p) (find-package "MAXIMA"))
+#+cmu (defun compiled-function-p (x)
+	(and (functionp x) (not (symbolp x))
+	     (not (eval:interpreted-function-p x))))
+
 (defprop compiled-function compiled-function-p ml-typep)
 (defprop extended-number extended-number-p ml-typep)
 (defprop fixnum fixnump ml-typep)
@@ -453,7 +484,7 @@ compiler:
   (declare (special *mread-prompt*))
   arg					;ignore
   (setf (fill-pointer *sharp-read-buffer* ) 0)
-  (cond ((eql #\$ (tyipeek t stream))(send stream :tyi)
+  (cond ((eql #\$ (tyipeek t stream))(tyi stream)
 	 (setq meval-flag nil)))
   (with-output-to-string
    (st *sharp-read-buffer*) 
@@ -471,7 +502,6 @@ compiler:
  
 
 (set-dispatch-macro-character  #\#  #\$ 'x$-cl-macro-read)
-
 (defvar *macsyma-readtable*)
 
 (defun find-lisp-readtable-for-macsyma ()
@@ -532,15 +562,22 @@ compiler:
 
 (defvar *reset-var* t)
 
+(defvar *variable-initial-values* (make-hash-table)
+  "Hash table containing all Maxima defmvar variables and their initial
+values")
+
 (defmacro defmvar (var &rest val-and-doc)
   "If *reset-var* is true then loading or eval'ing will reset value, otherwise like defvar"
   (cond ((> (length val-and-doc) 2)
 	 (setq val-and-doc (list (car val-and-doc) (second val-and-doc)))))
   `(progn
-	   #+lispm (si::record-source-file-name ',var 'defmvar)
-	  (defvar ,var ,@ val-and-doc)
-	  #+debug
-	  (maybe-reset ',var ',(if val-and-doc (list (car val-and-doc))))))
+    #+lispm (si::record-source-file-name ',var 'defmvar)
+    (unless (gethash ',var *variable-initial-values*)
+      (setf (gethash ',var *variable-initial-values*)
+	    ,(first val-and-doc)))
+    (defvar ,var ,@ val-and-doc)
+    #+debug
+    (maybe-reset ',var ',(if val-and-doc (list (car val-and-doc))))))
 
 (defun maybe-reset (var val-and-doc &aux val)
   (cond (*reset-var*
@@ -661,7 +698,7 @@ compiler:
 		 (t (setq string (format nil "~ve" (+ 4 $fpprec) symb)))))
 	 (setq string (string-left-trim " " string))
 	 )
-	#-gmp
+	#+(and gcl (not gmp))
 	((bignump symb)
 	 (let* ((big symb)
 		ans rem tem
@@ -797,6 +834,12 @@ compiler:
 	 (intern (string (aref strin (f- i 1)))))
 	(t nil)))
 
+(defun getchars (symb start end &aux strin)
+  (setq strin (string symb))
+  (cond ((and (<= start (length strin)) (> start 0))
+	 (intern (string (subseq strin (f- start 1) (f- end 1)))))
+	(t nil)))
+
 (defun ascii (n)
   (intern (string n)))
 
@@ -1142,7 +1185,7 @@ format:
    (let* ((n (file-length st))
 	  (ar (make-array n :element-type '#.(array-element-type "a")))
 	  )
-     (declare (type (array #.(array-element-type "a")) ar))
+     (declare (type (cl:array #.(array-element-type "a")) ar))
      (sloop for i below n 
 	    for tem = (read-char st nil)
 	    do 
