@@ -482,10 +482,11 @@ Maybe you want to use NOT EQUAL."))
       (let (sign-imag-errp limitp) (catch 'sign-imag-err ($sign x)))))
 
 (defmfun $sign (x)
-  (let (sign minus odds evens factored)
-       (sign01 (cond (limitp (restorelim x))
-		     ((not (free x '$%i)) ($rectform x))
-		     (t x)))))
+  (let ((x (specrepcheck x))
+	sign minus odds evens factored)
+    (sign01 (cond (limitp (restorelim x))
+		  ((not (free x '$%i)) ($rectform x))
+		  (t x)))))
 
 (defun sign01 (a)
   (let ((e (sign-prep a)))
@@ -591,13 +592,13 @@ Maybe you want to use NOT EQUAL."))
 	 (setq $askexp (lmul evens)
 	       sign (cdr (assol $askexp locals)))
 	 (do () (nil)
-	     (cond ((zl-MEMBER sign '($zero $z 0 0.0))
+	     (cond ((zl-MEMBER sign '($zero $z |$z| 0 0.0))
 		    (tdzero $askexp) (setq sign '$zero) (return t))
-		   ((memq sign '($pn $nonzero $n $nz $nonz $non0))
+		   ((memq sign '($pn $nonzero $n |$n| $nz $nonz $non0))
 		    (tdpn $askexp) (setq sign '$pos) (return t))
-		   ((memq sign '($pos $p $positive))
+		   ((memq sign '($pos $p |$p| $positive))
 		    (tdpos $askexp) (setq sign '$pos) (return t))
-		   ((memq sign '($neg $n $negative))
+		   ((memq sign '($neg $n |$n| $negative))
 		    (tdneg $askexp) (setq sign '$pos) (return t)))
 	     (setq sign (ask "Is  " $askexp "  zero or nonzero?")))
 	 (if minus (flip sign) sign))
@@ -609,13 +610,13 @@ Maybe you want to use NOT EQUAL."))
 			   ((eq '$pn sign) "  positive or negative?")
 			   (t "  positive, negative, or zero?")))
 		(ans (cdr (assol $askexp locals)))) (nil)
-	       (cond ((and (memq ans '($pos $p $positive))
+	       (cond ((and (memq ans '($pos $p |$p| $positive))
 			   (memq sign '($pz $pn $pnz)))
 		      (tdpos $askexp) (setq sign '$pos) (return t))
-		     ((and (memq ans '($neg $n $negative))
+		     ((and (memq ans '($neg $n |$n| $negative))
 			   (memq sign '($nz $pn $pnz)))
 		      (tdneg $askexp) (setq sign '$neg) (return t))
-		     ((and (zl-MEMBER ans '($zero $z 0 0.0))
+		     ((and (zl-MEMBER ans '($zero $z |$z| 0 0.0))
 			   (memq sign '($pz $nz $pnz)))
 		      (tdzero $askexp) (setq sign '$zero) (return t)))
 	       (setq ans (ask "Is  " $askexp dom)))
