@@ -188,35 +188,43 @@
 				       NIL 'MPAREN 'MPAREN 0 0))
 	    ,@ body)))
 
+(defvar *alt-display2d* nil)
+(defvar *alt-display1d* nil)
 (DEFMFUN DISPLA (FORM &aux #+kcl(form form))
   (IF (OR (NOT #.TTYOFF) #.WRITEFILEP)
       (cond #+Franz ($typeset (apply #'$photot (list form)))
-	    ($DISPLAY2D
-	     (LET ((DISPLAYP T)
-		   (LINEARRAY (IF DISPLAYP (MAKE-array 80.) LINEARRAY))
-		   (MRATP (CHECKRAT FORM))
-		   (#.WRITEFILEP #.WRITEFILEP)
-		   (MAXHT     1) (MAXDP   0) (WIDTH   0)
-		   (HEIGHT    0) (DEPTH   0) (LEVEL   0) (SIZE   2)
-		   (BREAK     0) (RIGHT   0) (LINES   1) BKPT
-		   (BKPTWD    0) (BKPTHT  1) (BKPTDP  0) (BKPTOUT 0)
-		   (BKPTLEVEL 0) IN-P
-		   (MOREFLUSH D-MOREFLUSH)
-		   MORE-^W
-		   (MOREMSG D-MOREMSG))
-	       (UNWIND-PROTECT
-		(PROGN (SETQ FORM (DIMENSION FORM
-					     NIL 'MPAREN 'MPAREN 0 0))
-		       (CHECKBREAK FORM WIDTH)
-		       (OUTPUT FORM (IF (AND (NOT $LEFTJUST) (= 2 LINES))
-					(f- LINEL (f- WIDTH BKPTOUT))
-					0))
-		       (IF (AND SMART-TTY (NOT (AND SCROLLP (NOT $CURSORDISP)))
-				(> (CAR (CURSORPOS)) (f- TTYHEIGHT 3)))
-			   (LET (#.writefilep) (MTERPRI))))
-	     ;; make sure the linearray gets cleared out.
-	     (CLEAR-LINEARRAY))))
-	    (T (LINEAR-DISPLA FORM)))))
+	    ($DISPLAY2D 
+	     (cond
+	      (*alt-display2d* (apply *alt-display2d* form ()))
+	      (t 
+	       (LET ((DISPLAYP T)
+		     (LINEARRAY (IF DISPLAYP (MAKE-array 80.) LINEARRAY))
+		     (MRATP (CHECKRAT FORM))
+		     (#.WRITEFILEP #.WRITEFILEP)
+		     (MAXHT     1) (MAXDP   0) (WIDTH   0)
+		     (HEIGHT    0) (DEPTH   0) (LEVEL   0) (SIZE   2)
+		     (BREAK     0) (RIGHT   0) (LINES   1) BKPT
+		     (BKPTWD    0) (BKPTHT  1) (BKPTDP  0) (BKPTOUT 0)
+		     (BKPTLEVEL 0) IN-P
+		     (MOREFLUSH D-MOREFLUSH)
+		     MORE-^W
+		     (MOREMSG D-MOREMSG))
+		    (UNWIND-PROTECT
+		     (PROGN (SETQ FORM (DIMENSION FORM
+						  NIL 'MPAREN 'MPAREN 0 0))
+			    (CHECKBREAK FORM WIDTH)
+			    (OUTPUT FORM (IF (AND (NOT $LEFTJUST) (= 2 LINES))
+					     (f- LINEL (f- WIDTH BKPTOUT))
+					     0))
+			    (IF (AND SMART-TTY (NOT (AND SCROLLP (NOT $CURSORDISP)))
+				     (> (CAR (CURSORPOS)) (f- TTYHEIGHT 3)))
+				(LET (#.writefilep) (MTERPRI))))
+		     ;; make sure the linearray gets cleared out.
+		     (CLEAR-LINEARRAY))))))
+	    (T 
+	     (cond
+	      (*alt-display1d* (apply *alt-display1d* form ()))
+	      (t (LINEAR-DISPLA FORM)))))))
 
 (defun transform-extends (x)
   (cond (($extendp x)
@@ -773,7 +781,7 @@
 
 (DISPLA-DEF %LSUM   DIM-%LSUM 110.)
 (DEFUN DIM-%LSUM (FORM RESULT) (DSUMPROD FORM RESULT '(D-SUMSIGN) 4 3 2))
-(DEFUN DIM-%SUM (FORM RESULT) (DSUMPROD FORM RESULT '(D-SUMSIGN) 10 3 2))
+(DEFUN DIM-%SUM (FORM RESULT) (DSUMPROD FORM RESULT '(D-SUMSIGN) 4 3 2))
 
 (DEFUN DIM-%LIMIT (FORM RESULT)
   (PROG ((W 0) (H 0) (D 0) DUMMY) (DECLARE (FIXNUM W H D))
@@ -1810,6 +1818,8 @@
 (defvar *big-chunk-size*  9)
 (setq *tentochunksize* (expt 10 *big-chunk-size*))
 
+;; Defined in commac.lisp.
+#+nil
 (defun exploden (symb &aux string)
   (cond ((symbolp symb)(setq string (symbol-name symb)))
         ((floatp symb)
@@ -1821,6 +1831,7 @@
 		 (t (setq string (format nil "~ve" (+ 4 $fpprec) symb)))))
 	 (setq string (string-left-trim " " string))
 	 )
+	#+(and gcl (not gmp))
 	((bignump symb)
 	 (let* ((big symb)
 		ans rem tem
