@@ -287,10 +287,10 @@
 		(IF (SETQ VAL (ARGS ITEM))
 		    (FASPRIN `(ARGS (QUOTE ,RENAME) (QUOTE ,VAL))))
 		(PROPSCHK ITEM RENAME 'TRANSLATED))
-	  (WHEN (AND (SETQ VAL (GETL ITEM '(A-EXPR FEXPR TRANSLATED-MMACRO))) 
-		     (GET ITEM 'TRANSLATED))
-		(DSKDEFPROP RENAME (CADR VAL) (CAR VAL))
-		(PROPSCHK ITEM RENAME 'TRANSLATED))
+;; 	  (WHEN (AND (SETQ VAL (GETL ITEM '(A-EXPR FEXPR TRANSLATED-MMACRO))) 
+;; 		     (GET ITEM 'TRANSLATED))
+;; 		(DSKDEFPROP RENAME (CADR VAL) (CAR VAL))
+;; 		(PROPSCHK ITEM RENAME 'TRANSLATED))
 	  (WHEN (SETQ VAL (GET ITEM 'OPERATORS))
 		(DSKDEFPROP RENAME VAL 'OPERATORS)
 		(WHEN (SETQ VAL (GET ITEM 'RULES))
@@ -359,7 +359,7 @@
 (DEFUN MPROPSCHK (ITEM RENAME FILE STFL)
  (DO ((PROPS (CDR (OR (GET ITEM 'MPROPS) '(NIL))) (CDDR PROPS)) (VAL))
      ((NULL PROPS))
-     (COND ((OR (MEMQ (CAR PROPS) '(TRACE TRACE-TYPE TRACE-LEVEL))
+     (COND ((OR (MEMQ (CAR PROPS) '(TRACE TRACE-TYPE TRACE-LEVEL trace-oldfun))
 		(MFILEP (SETQ VAL (CADR PROPS)))
 		(AND (EQ (CAR PROPS) 'T-MFEXPR) (NOT (GET ITEM 'TRANSLATED)))))
 	   ((NOT (MEMQ (CAR PROPS) '(HASHAR ARRAY)))
