@@ -25,6 +25,7 @@
 ;; and compile time.  These variables should probably be set up in a prelude
 ;; file so they can be accessible to all Macsyma files.
 
+#+nil
 #.(SETQ MACHINE-FIXNUM-PRECISION
 	#+(OR PDP10 H6180)   36.
 	#+cl (integer-length most-positive-fixnum)
@@ -32,25 +33,37 @@
 	#+NIL		     30.
 	#+Franz		     32.
 
+#|
 	MACHINE-MANTISSA-PRECISION
 	#+(OR PDP10 H6180)   27.
 	#+cl(integer-length (integer-decode-float most-positive-double-float))
 	;#+LISPM		     32.
 	#+(OR NIL Franz)     56.	;double-float.  Long would be 113.
-
+|#
 	;; Not used anymore, but keep it around anyway in case
 	;; we need it later.
 
 	MACHINE-EXPONENT-PRECISION
 	#+(OR PDP10 H6180)    8.
 	#+cl
-	(integer-length (multiple-value-bind
-		  (a b) (integer-decode-float most-positive-double-float)
-		  b))
+	(integer-length
+	 (multiple-value-bind (a b)
+	     (integer-decode-float most-positive-double-float)
+	   b))
 	;#+LISPM		     11.
 	#+(OR NIL Franz)      8.	;Double float.  Long would be 15.
 	)
 
+(eval-when (compile load eval)
+(defconstant +machine-fixnum-precision+
+  (integer-length most-positive-fixnum))
+
+;; Hmm, this doesn't seem to be used anywhere, but we leave here anyway. 
+(defconstant +machine-exponent-precision+
+  (integer-length (multiple-value-bind (a b)
+		      (integer-decode-float most-positive-double-float)
+		    b)))
+)
 ;; External variables
 
 (DEFMVAR $FLOAT2BF NIL
@@ -284,12 +297,14 @@ One extra decimal digit in actual representation for rounding purposes.")
     ;;and then convert it to a floating point fraction.
     (SETQ MANTISSA (QUOTIENT (FPROUND MANTISSA)
 			     #.(EXPT 2.0 MACHINE-MANTISSA-PRECISION)))
-    ;;Multiply the mantissa by the exponent portion.  I'm not sure
-    ;;why the exponent computation is so complicated.
-    (SETQ PRECISION
-	  (ERRSET (TIMES MANTISSA (EXPT 2.0 (f+ EXPONENT (MINUS PRECISION) *M
-					       #.MACHINE-MANTISSA-PRECISION)))
-		  NIL))
+    ;; Multiply the mantissa by the exponent portion.  I'm not sure
+    ;; why the exponent computation is so complicated. Using
+    ;; scale-float will prevent possible overflow unless the result
+    ;; really would.
+    (setq precision
+	  (errset (scale-float mantissa (f+ exponent (minus precision) *m
+					    #.machine-mantissa-precision))
+		  nil))
     (IF PRECISION
 	(CAR PRECISION)
 	(MERROR "Floating point overflow in converting ~:M to flonum" L))))
@@ -573,11 +588,22 @@ One extra decimal digit in actual representation for rounding purposes.")
 			    FPPREC))
 	  (RETURN (FPSHIFT L (MINUS *M)))))))
 
+;; Compute (* L (expt d n)) where D is 2 or 10 depending on
+;; *decfp. Throw away an fractional part by truncating to zero.
 (DEFUN FPSHIFT (L N) 
-       (COND ((NULL *DECFP) (BIGLSH L N))
-	     ((GREATERP N 0.) (TIMES L (EXPT 10. N)))
-	     ((LESSP N 0.) (QUOTIENT L (EXPT 10. (MINUS N))))
-	     (T L))) 
+  (COND ((NULL *DECFP)
+	 (cond ((and (minusp n) (minusp l))
+		;; Left shift of negative number requires some
+		;; care. (That is, (truncate l (expt 2 n)), but use
+		;; shifts instead.)
+		(- (ash (- l) n)))
+	       (t
+		(ash l n))))
+	((GREATERP N 0.)
+	 (TIMES L (EXPT 10. N)))
+	((LESSP N 0.)
+	 (QUOTIENT L (EXPT 10. (MINUS N))))
+	(T L)))
 
 ;; Bignum LSH -- N is assumed (and declared above) to be a fixnum.
 ;; This isn't really LSH, since the sign bit isn't propagated when
@@ -594,13 +620,13 @@ One extra decimal digit in actual representation for rounding purposes.")
    ;; In MacLisp, the result is undefined if the magnitude of the
    ;; second argument is greater than 36.
    ((AND (NOT (BIGP X))
-	 (< N #.(f- MACHINE-FIXNUM-PRECISION))) 0)
+	 (< N #.(f- +machine-fixnum-precision+))) 0)
    ;; Either we are shifting a fixnum to the right, or shifting
    ;; a fixnum to the left, but not far enough left for it to become
    ;; a bignum.
    ((AND (NOT (BIGP X)) 
 	 (OR (<= N 0)
-	     (< (PLUS (HAULONG X) N) #.MACHINE-FIXNUM-PRECISION)))
+	     (< (PLUS (HAULONG X) N) #.+machine-fixnum-precision+)))
     ;; The form which follows is nearly identical to (ASH X N), however
     ;; (ASH -100 -20) = -1, whereas (BIGLSH -100 -20) = 0.
     (IF (>= X 0)
@@ -616,7 +642,7 @@ One extra decimal digit in actual representation for rounding purposes.")
    ((= N 0) X)
    ;; Isn't this the kind of optimization that compilers are
    ;; supposed to make?
-   ((< N #.(f1- MACHINE-FIXNUM-PRECISION)) (TIMES X (LSH 1 N)))
+   ((< N #.(f1- +machine-fixnum-precision+)) (TIMES X (LSH 1 N)))
    (T (TIMES X (EXPT 2 N)))))
 
 
@@ -996,16 +1022,46 @@ One extra decimal digit in actual representation for rounding purposes.")
 		   (T (LIST '(%LOG) (CAR A)))))
 	    (PLUS 2 FPPREC)))
    (COND ((NUMBERP (CAR A))
-	  (SETQ A (LIST (FPROUND (CAR A)) (PLUS -2 *M (CADR A))))
+	  (SETQ A
+		(if (zerop (car a))
+		    (list 0 0)
+		    (LIST (FPROUND (CAR A)) (PLUS -2 *M (CADR A)))))
 	  (SETQ A (BCONS A))))
    (COND (MINUS (ADD A (MUL '$%I ($BFLOAT '$%PI)))) (T A)))
   NIL)) 
 
+;;; Computes the log of a bigfloat number.
+;;;
+;;; Uses the series
+;;;
+;;; log(1+x) = sum((x/(x+2))^(2*n+1)/(2*n+1),n,0,inf);
+;;;
+;;;
+;;;                  INF      x   2 n + 1
+;;;                  ====  (-----)
+;;;                  \      x + 2
+;;;          =  2     >    --------------
+;;;                  /        2 n + 1
+;;;                  ====
+;;;                  n = 0
+;;;
+;;;
+;;; which converges for x > 0.
+;;;
+;;; Note that FPLOG is given 1+X, not X.
+;;;
+;;; However, to aid convergence of the series, we scale 1+x until 1/e
+;;; < 1+x <= e.
+;;;
 (DEFUN FPLOG (X) 
-       (PROG (OVER TWO ANS OLDANS TERM E) 
+       (PROG (OVER TWO ANS OLDANS TERM E sum) 
 	     (IF (NOT (GREATERP (CAR X) 0))
 		 (MERROR "Non-positive argument to FPLOG"))
-	     (SETQ E (FPE) OVER (FPQUOTIENT (FPONE) E) ANS 0)
+	     (SETQ E (FPE)
+		   OVER (FPQUOTIENT (FPONE) E)
+		   ANS 0)
+	     ;; Scale X until 1/e < X <= E.  ANS keeps track of how
+	     ;; many factors of E were used.  Set X to NIL if X is E.
 	     (DO () (NIL)
 		 (COND ((EQUAL X E) (SETQ X NIL) (RETURN NIL))
 		       ((AND (FPLESSP X E) (FPLESSP OVER X))
@@ -1016,21 +1072,29 @@ One extra decimal digit in actual representation for rounding purposes.")
 		       (T (SETQ ANS (ADD1 ANS))
 			  (SETQ X (FPQUOTIENT X e)))))
 	     (COND ((NULL X) (RETURN (INTOFP (ADD1 ANS)))))
-	     (SETQ X (FPDIFFERENCE  X (FPONE)) ANS (INTOFP ANS))
+	     ;; Prepare X for the series.  The series is for 1 + x, so
+	     ;; get x from our X.  TERM is (x/(x+2)).  X becomes
+	     ;; (x/(x+2))^2.
+	     (SETQ X (FPDIFFERENCE  X (FPONE))
+		   ANS (INTOFP ANS))
 	     (SETQ 
 	      X
 	      (FPEXPT (SETQ TERM
 			    (FPQUOTIENT X (FPPLUS X (SETQ TWO (INTOFP 2)))))
 		      2))
-	     (DO ((N 0 (f1+ N))) ((EQUAL ANS OLDANS))
-	      (SETQ OLDANS ANS)
-	      (SETQ 
-	       ANS
-	       (FPPLUS
-		ANS
-		(FPTIMES* TWO (FPQUOTIENT TERM (INTOFP (f1+ (f* 2 N)))))))
+	     ;; Sum the series until the sum (in ANS) doesn't change
+	     ;; anymore.
+	     (setq sum (intofp 0))
+	     (DO ((N 1 (+ N 2)))
+		 ((EQUAL sum OLDANS))
+	      (SETQ OLDANS sum)
+	      (SETQ sum
+		    (FPPLUS
+		     sum
+		     (FPQUOTIENT TERM (INTOFP N))))
 	      (SETQ TERM (FPTIMES* TERM X)))
-	     (RETURN ANS))) 
+	     
+	     (return (fpplus ANS (fptimes* two sum)))))
 
 (DEFUN MABSBIGFLOAT (L) 
        (PROG (R) 
