@@ -12,7 +12,7 @@
 (macsyma-module grind)
 
 (declare-top (GENPREFIX GRI)
-	 (SPECIAL LOP ROP STRING CHRPS $ALIASES ALIASLIST LINEL)
+	 (SPECIAL LOP ROP *GRIND-CHARLIST* CHRPS $ALIASES ALIASLIST LINEL)
 	 (FIXNUM (CHRCT))
 	 (*EXPR LBP RBP))
 
@@ -43,7 +43,8 @@
 ;Update from F302 --gsb
 (DEFMSPEC $GRIND (X) (SETQ X (CDR X))
   (LET (Y)
-    (IF (NOT (ZEROP (CHARPOS T))) (MTERPRI))
+    #+nocp(fresh-line)
+    #-nocp(IF (NOT (ZEROP (CHARPOS T))) (MTERPRI))
     (COND ((OR (NULL X) (CDR X)) (WNA-ERR '$GRIND))
 	  ((SYMBOLP (SETQ X (STRMEVAL (CAR X))))
 	   (SETQ X ($VERBIFY X))
@@ -79,7 +80,8 @@
 
 (defun i-$grind (x)
   (LET (Y)
-    (IF (NOT (ZEROP (CHARPOS T))) (MTERPRI))
+    #+nocp(fresh-line)
+    #-nocp(IF (NOT (ZEROP (CHARPOS T))) (MTERPRI))
     (COND  ((SYMBOLP (SETQ X (STRMEVAL  X)))
 	    (SETQ X ($VERBIFY X))
 	    (COND ((SETQ Y (MGET X 'MEXPR))
@@ -125,9 +127,9 @@
   (DO () ((< N 1)) (TYO #\SPACE OUT) (SETQ N (f1- N))))
 
 (DEFUN STRGRIND (X)
-  (LET (STRING (CHRPS 0))
+  (LET (*GRIND-CHARLIST* (CHRPS 0))
     (STRPRINT (MSIZE X NIL NIL 'MPAREN 'MPAREN))
-    (NREVERSE STRING)))
+    (NREVERSE *GRIND-CHARLIST*)))
 
 (DEFUN STRPRINT (X)
   (COND ((ATOM X) (STYO X))
@@ -149,13 +151,13 @@
 		  (t (SETQ CHRPS 0) (STERPRI) (STYOTBSP I)))
 		 (STRPRINT (CAR L)))))))
 
-(DEFUN STYO (X) (SETQ STRING (CONS X STRING) CHRPS (f1+ CHRPS)))
+(DEFUN STYO (X) (SETQ *GRIND-CHARLIST* (CONS X *GRIND-CHARLIST*) CHRPS (f1+ CHRPS)))
 
-(DEFUN STERPRI () (SETQ STRING (CONS #\NEWLINE STRING) CHRPS 0))
+(DEFUN STERPRI () (SETQ *GRIND-CHARLIST* (CONS #\NEWLINE *GRIND-CHARLIST*) CHRPS 0))
 
 (DEFUN STYOTBSP (N) (DECLARE (FIXNUM N)) (SETQ CHRPS N)
-  (DO () ((< N 8)) (SETQ STRING (CONS #\TAB STRING) N (f- N 8)))
-  (DO () ((< N 1)) (SETQ STRING (CONS #\SPACE STRING) N (f1- N))))
+  (DO () ((< N 8)) (SETQ *GRIND-CHARLIST* (CONS #\TAB *GRIND-CHARLIST*) N (f- N 8)))
+  (DO () ((< N 1)) (SETQ *GRIND-CHARLIST* (CONS #\SPACE *GRIND-CHARLIST*) N (f1- N))))
 
 (DEFMFUN MSTRING (X)
   (NREVERSE (STRING1 (MSIZE X NIL NIL 'MPAREN 'MPAREN) NIL)))
