@@ -1,7 +1,9 @@
+(in-package "MAXIMA")
 ;; TeX-printing
 ;; (c) copyright 1987, Richard J. Fateman
-
-(in-package "MAXIMA")
+;; small corrections and additions: Andrey Grozin, 2001
+;; additional additions: Judah Milgram (JM), September 2001
+;; additional corrections: Barton Willis (BLW), October 2001
 
 ;; Usage: tex(d8,"/tmp/foo.tex"); tex(d10,"/tmp/foo.tex"); ..
 ;; to append lines d8 and d10 to the tex file.  If given only
@@ -15,7 +17,7 @@
 ;; You have my permission to put it in NESC or give it to anyone
 ;; else who might be interested in it....
 
-;; source language: 
+;; source language:
 ;; There are changes by wfs to allow use inside MAXIMA which runs
 ;; in COMMON LISP.  For original FRANZ LISP version contact rfw.
 
@@ -38,14 +40,14 @@
 ;; with appropriate substitutions and recognition of the
 ;; infix / prefix / postfix / matchfix relations on symbols. Various
 ;; changes are made to this so that TeX will like the results.
-;; It is important to understand the binding powers of the operators 
+;; It is important to understand the binding powers of the operators
 ;; in Macsyma, in mathematics, and in TeX so that parentheses will
 ;; be inserted when necessary. Because TeX has different kinds of
 ;; groupings (e.g. in superscripts, within sqrts), not all
 ;; parentheses are explicitly need.
 
 ;;  Instructions:
-;; in macsyma, type tex(<expression>);  or tex(<label>); or 
+;; in macsyma, type tex(<expression>);  or tex(<label>); or
 ;; tex(<expr-or-label>, <file-name>);  In the case of a label,
 ;; a left-equation-number will be produced.
 ;; in case a file-name is supplied, the output will be sent
@@ -56,8 +58,8 @@
 #+franz
 ($bothcases t) ;; allow alpha and Alpha to be different
 (declare-top
-	 (special lop rop ccol $gcprint texport $labels $inchar 
-		  vaxima-main-dir 
+	 (special lop rop ccol $gcprint texport $labels $inchar
+		  vaxima-main-dir
 		  )
 	 (*expr tex-lbp tex-rbp))
 
@@ -80,7 +82,13 @@
 	   )
 	  (t (apply 'tex1  args)))))
 
-
+(defun quote-% (sym)
+  (let ((strsym (string sym)))
+    (cond ((position (character "%") strsym)
+	   (let ((pos (position (character "%") strsym)))
+	     (concatenate 'string (subseq strsym 0 pos) "\\%" 
+			  (quote-% (subseq strsym (+ pos 1))))))
+	  (t strsym))))
 
 (defun tex1 (mexplabel &optional filename ) ;; mexplabel, and optional filename
   (prog (mexp  texport $gcprint ccol x y itsalabel)
@@ -92,17 +100,18 @@
 	;; collect the file-name, if any, and open a port if needed
 	(setq texport (cond((null filename) *standard-output* ); t= output to terminal
 			   (t
-			     (open (stripdollar filename)
+			     (open (string (stripdollar filename))
 				   :direction :output
-				   :if-exists :append))))
+				   :if-exists :append
+				   :if-does-not-exist :create))))
 	;; go back and analyze the first arg more thoroughly now.
 	;; do a normal evaluation of the expression in macsyma
-	(setq mexp (meval mexplabel)) 
+	(setq mexp (meval mexplabel))
 	(cond ((memq mexplabel $labels); leave it if it is a label
 	       (setq mexplabel (concat "(" (stripdollar mexplabel) ")"))
 	       (setq itsalabel t))
 	      (t (setq mexplabel nil)));flush it otherwise
-	
+
 	;; maybe it is a function?
 	(cond((symbolp (setq x mexp)) ;;exclude strings, numbers
 	      (setq x ($verbify x))
@@ -111,25 +120,34 @@
 		    ((setq y (mget x 'mmacro))
 		     (setq mexp (list '(mdefmacro) (cons (list x) (cdadr y)) (caddr y))))
 		    ((setq y (mget x 'aexpr))
-		     (setq mexp (list '(mdefine) (cons (list x 'array) (cdadr y)) (caddr y))))))) 
+		     (setq mexp (list '(mdefine) (cons (list x 'array) (cdadr y)) (caddr y)))))))
 	(cond ((and (null(atom mexp))
 		    (memq (caar mexp) '(mdefine mdefmacro)))
+               (if mexplabel (setq mexplabel (quote-% mexplabel)))
 	       (format texport "|~%" ) ;delimit with |marks
 	       (cond (mexplabel (format texport "~a " mexplabel)))
 	       (mgrind mexp texport) ;write expression as string
 	       (format texport ";|~%"))
-	      
 	      ((and
 		itsalabel ;; but is it a user-command-label?
-		(eq (getchar $inchar 2) (getchar mexplabel 2)))
+                (<= (length (string $inchar)) (length (string mexplabel)))
+		(eq (getchars $inchar 2 (1+ (length (string $inchar))))
+		    (getchars mexplabel 2 (1+ (length (string $inchar)))))
+                ;; Check to make sure it isn't an outchar in disguise
+                (not
+                 (and
+                  (<= (length (string $outchar)) (length (string mexplabel)))
+                  (eq (getchars $outchar 2 (1+ (length (string $outchar))))
+                      (getchars mexplabel 2 (1+ (length (string $outchar))))))))
 	       ;; aha, this is a C-line: do the grinding:
 	       (format texport "~%|~a " mexplabel) ;delimit with |marks
 	       (mgrind mexp texport) ;write expression as string
 	       (format texport ";|~%"))
-	      
-	      (t ; display the expression for TeX now:
+	      (t 
+                 (if mexplabel (setq mexplabel (quote-% mexplabel)))
+                 ; display the expression for TeX now:
 		 (myprinc "$$")
-		 (mapc #'myprinc 
+		 (mapc #'myprinc
 		       ;;initially the left and right contexts are
 		       ;; empty lists, and there are implicit parens
 		       ;; around the whole expression
@@ -141,20 +159,20 @@
 		      (close texport)))
 	(return mexplabel)))
 
-;;; myprinc is an intelligent low level printing routine.  it keeps track of 
+;;; myprinc is an intelligent low level printing routine.  it keeps track of
 ;;; the size of the output for purposes of allowing the TeX file to
-;;; have a reasonable line-line. myprinc will break it at a space 
+;;; have a reasonable line-line. myprinc will break it at a space
 ;;; once it crosses a threshold.
 ;;; this has nothign to do with breaking the resulting equations.
- 
+
 ;-      arg:    chstr -  string or number to princ
 ;-      scheme: This function keeps track of the current location
 ;-              on the line of the cursor and makes sure
 ;-              that a value is all printed on one line (and not divided
 ;-              by the crazy top level os routines)
- 
+
 (defun myprinc (chstr)
-       (prog (chlst) 
+       (prog (chlst)
               (cond ((greaterp (plus (length (setq chlst (exploden chstr)))
                                  ccol)
                            70.)
@@ -188,14 +206,22 @@
 	      (t (tex-function x l r nil))))
 
 (defun tex-atom (x l r) ;; atoms: note: can we lose by leaving out {}s ?
-  (append l 
-	  (list (cond ((numberp x)(texnumformat x) )
+  (append l
+	  (list (cond ((numberp x) (texnumformat x))
 		      ((and (symbolp x) (get x 'texword)))
+                      ((stringp x) (tex-string x))
+                      ((characterp x) (tex-char x))
 		      (t (tex-stripdollar x))))
-	  
 	  r))
 
+(defun tex-string (x)
+  (cond ((equal x "") "")
+    ((eql (elt x 0) #\\) x)
+    (t (concatenate 'string "\\mbox{{}" x "{}}"))))
 
+(defun tex-char (x)
+  (if (eql x #\|) "\\mbox{\\verb/|/}"
+    (concatenate 'string "\\mbox{\\verb|" (string x) "|}")))
 
 (defvar *tex-translations* nil)
 ;; '(("AB" . "a")("X" . "x")) would cause  AB12 and X3 C4 to print a_{12} and x_3 C_4
@@ -236,8 +262,16 @@
 		  (vector-push #\} tem))))
     (intern tem)))
 
-    
-#+cmu
+;; A.G. 2001: I prefer the following version:
+;(defun tex-stripdollar (sym)
+;  (or (symbolp sym) (return-from tex-stripdollar sym))
+;  (let* ((name (symbol-name sym))
+;      (pname (if (eql (elt name 0) #\$) (subseq name 1) name))
+;      (l (length pname)))
+;    (cond
+;      ((eql l 1) pname)
+;      (t (concatenate 'string "\\mathrm{" pname "}")))))
+
 (defun strcat (&rest args)
   (apply #'concatenate 'string (mapcar #'string args)))
 
@@ -261,40 +295,40 @@
 			  (apply #'strcat (cdr exponent))
 			  "}")))))))
 
-(defun tex-paren (x l r) 
+(defun tex-paren (x l r)
   (tex x (append l '("\\left(")) (cons "\\right)" r) 'mparen 'mparen))
 
 (defun tex-array (x l r)
   (let ((f))
-       (if (eq 'mqapply (caar x))
-	   (setq f (cadr x) 
-		 x (cdr x))
-	   (setq f (caar x)))
-       (setq l (tex (texword f) l nil lop 'mfunction)
-	     
-	     r (nconc (tex-list (cdr x) nil (list "}") ",") r)) 
-       (nconc l (list "_{") r  )))
+    (if (eq 'mqapply (caar x))
+	(setq f (cadr x)
+	      x (cdr x)
+	      l (tex f (append l (list "\\left(")) (list "\\right)") 'mparen 'mparen))
+      (setq f (caar x)
+	    l (tex (texword f) l nil lop 'mfunction)))
+    (setq
+     r (nconc (tex-list (cdr x) nil (list "}") ",") r))
+    (nconc l (list "_{") r  )))
 
 ;; we could patch this so sin x rather than sin(x), but instead we made sin a prefix
 ;; operator
 
 (defun tex-function (x l r op) op
-	(setq l (tex (texword (caar x)) ;;see if a function has a special symbol
-		     l 
-		     (list "\\left(" ) 'mparen 'mparen)
-		     r (tex-list (cdr x) nil (cons "\\right)" r) ","))
+	(setq l (tex (texword (caar x)) l nil 'mparen 'mparen)
+	      r (tex (cons '(mprogn) (cdr x)) nil r 'mparen 'mparen))
 	(nconc l r))
+
 ;; set up a list , separated by symbols (, * ...)  and then tack on the
 ;; ending item (e.g. "]" or perhaps ")"
 
 (defun tex-list (x l r sym)
-  (if (null x) nil
+  (if (null x) r
       (do ((nl))
 	  ((null (cdr x))
-	   (setq nl (nconc nl (tex (car x)  l r lop rop)))
+	   (setq nl (nconc nl (tex (car x)  l r 'mparen 'mparen)))
 	   nl)
-	  (setq nl (nconc nl (tex (car x)  l (list sym)   lop rop))
-		  x (cdr x) 
+	  (setq nl (nconc nl (tex (car x)  l (list sym) 'mparen 'mparen))
+		  x (cdr x)
 		  l nil))))
 
 (defun tex-prefix (x l r)
@@ -303,31 +337,32 @@
 (defun tex-infix (x l r)
   ;; check for 2 args
   (if (or (null (cddr x)) (cdddr x)) (wna-err (caar x)))
-  
   (setq l (tex (cadr x) l nil lop (caar x)))
   (tex (caddr x) (append l (texsym (caar x))) r (caar x) rop))
-  
-
 
 (defun tex-postfix (x l r)
   (tex (cadr x) l (append (texsym (caar x)) r) lop (caar x)))
 
-(defun tex-nary (x l r) (texnary x l r (texsym (caar x))))
+(defun tex-nary (x l r)
+  (let* ((op (caar x)) (sym (texsym op)) (y (cdr x)) (ext-lop lop) (ext-rop rop))
+    (cond ((null y)       (tex-function x l r t)) ; this should not happen
+          ((null (cdr y)) (tex-function x l r t)) ; this should not happen, too
+          (t (do ((nl) (lop ext-lop op) (rop op (if (null (cdr y)) ext-rop op)))
+                 ((null (cdr y)) (setq nl (nconc nl (tex (car y)  l r lop rop))) nl)
+	         (setq nl (nconc nl (tex (car y)  l (list sym)   lop rop))
+		       y (cdr y)
+		       l nil))))))
 
 (defun tex-nofix (x l r) (tex (caar x) l r (caar x) rop))
 
 (defun tex-matchfix (x l r)
-  (setq l (append l (car(texsym (caar x))) )
+  (setq l (append l (car (texsym (caar x))))
 	;; car of texsym of a matchfix operator is the lead op
-	r (append (cdr (texsym (caar x))) r) 
+	r (append (cdr (texsym (caar x))) r)
 	;; cdr is the trailing op
 	x (tex-list (cdr x) nil r ","))
   (append l x))
 
-(defun texnary (x l r dissym)
- (cond ((null (cddr x)) (tex-function x l r t))
-       (t (tex-list (cdr x) l r dissym))))
-
 (defun texsym (x) (or (get x 'texsym) (get x 'strsym)(get x 'dissym)
 		      (stripdollar x)))
 
@@ -337,12 +372,14 @@
 
 (defun tex-bigfloat (x l r) (fpformat x))
 
-(defprop mprog "{\\bf block\\>}" texword)
-(defprop %erf "{\\rm erf}" texword)
-(defprop $erf "{\\rm erf}" texword) ;; etc for multicharacter names
+(defprop mprog "\\mathbf{block}\\;" texword)
+(defprop %erf "\\mathrm{erf}" texword)
+(defprop $erf "\\mathrm{erf}" texword) ;; etc for multicharacter names
+(defprop $true  "\\mathbf{true}"  texword)
+(defprop $false "\\mathbf{false}" texword)
 
 (defprop mprogn tex-matchfix tex) ;; mprogn is (<progstmnt>, ...)
-(defprop mprogn (("(") ")") texsym)
+(defprop mprogn (("\\left(") "\\right)") texsym)
 
 (defprop mlist tex-matchfix tex)
 (defprop mlist (("\\left[ ")" \\right] ") texsym)
@@ -358,17 +395,37 @@
 	r (tex-list (cddr x) nil (cons ")" r) ","))
   (append l r));; fixed 9/24/87 RJF
 
-;; simple trial data
-(setf (get '$omega 'texword) "\\omega")
-;; alternative syntax for above..
-(defprop $Omega "\\Omega " texword)
 (defprop $%i "i" texword)
 (defprop $%pi "\\pi" texword)
 (defprop $%e "e" texword)
 (defprop $inf "\\infty " texword)
 (defprop $minf " -\\infty " texword)
-(defprop %laplace "{\\cal L}" texword)
-(defprop $alpha "\\alpha" texword) ;; etc
+(defprop %laplace "\\mathcal{L}" texword)
+(defprop $alpha "\\alpha" texword)
+(defprop $beta "\\beta" texword)
+(defprop $gamma "\\gamma" texword)
+(defprop %gamma "\\Gamma" texword)
+(defprop $%gamma "\\gamma" texword)
+(defprop $delta "\\delta" texword)
+(defprop $epsilon "\\varepsilon" texword)
+(defprop $zeta "\\zeta" texword)
+(defprop $eta "\\eta" texword)
+(defprop $theta "\\vartheta" texword)
+(defprop $iota "\\iota" texword)
+(defprop $kappa "\\varkappa" texword)
+;(defprop $lambda "\\lambda" texword)
+(defprop $mu "\\mu" texword)
+(defprop $nu "\\nu" texword)
+(defprop $xi "\\xi" texword)
+(defprop $pi "\\pi" texword)
+(defprop $rho "\\rho" texword)
+(defprop $sigma "\\sigma" texword)
+(defprop $tau "\\tau" texword)
+(defprop $upsilon "\\upsilon" texword)
+(defprop $phi "\\varphi" texword)
+(defprop $chi "\\chi" texword)
+(defprop $psi "\\psi" texword)
+(defprop $omega "\\omega" texword)
 
 (defprop mquote tex-prefix tex)
 (defprop mquote ("'") texsym)
@@ -394,6 +451,11 @@
 (defprop mdefmacro 180. tex-lbp)
 (defprop mdefmacro 20. tex-rbp)
 
+(defprop marrow tex-infix tex)
+(defprop marrow ("\\rightarrow ") texsym)
+(defprop marrow 25 tex-lbp)
+(defprop marrow 25 tex-rbp)
+
 (defprop mfactorial tex-postfix tex)
 (defprop mfactorial ("!") texsym)
 (defprop mfactorial 160. tex-lbp)
@@ -402,55 +464,58 @@
 (defprop mexpt 140. tex-lbp)
 (defprop mexpt 139. tex-rbp)
 
+(defprop %sum 110. tex-rbp)  ;; added by BLW, 1 Oct 2001
+(defprop %product 115. tex-rbp) ;; added by BLW, 1 Oct 2001
+
 ;; insert left-angle-brackets for mncexpt. a^<n> is how a^^n looks.
 (defun tex-mexpt (x l r)
   (let((nc (eq (caar x) 'mncexpt))); true if a^^b rather than a^b
      ;; here is where we have to check for f(x)^b to be displayed
      ;; as f^b(x), as is the case for sin(x)^2 .
-     ;; which should be sin^2 x rather than (sin x)^2 or (sin(x))^2. 
+     ;; which should be sin^2 x rather than (sin x)^2 or (sin(x))^2.
      ;; yet we must not display (a+b)^2 as +^2(a,b)...
      ;; or (sin(x))^(-1) as sin^(-1)x, which would be arcsine x
      (cond ;; this whole clause
-	   ;;should be deleted if this hack is unwanted and/or the
-	   ;;time it takes is of concern.
-	   ;;it shouldn't be too expensive.
+	   ;; should be deleted if this hack is unwanted and/or the
+	   ;; time it takes is of concern.
+	   ;; it shouldn't be too expensive.
 	   ((and (eq (caar x) 'mexpt) ; don't do this hack for mncexpt
-		 (let* 
+		 (let*
 		  ((fx (cadr x)); this is f(x)
-		   (f (and (not(atom fx)) (atom (caar fx))(caar fx))); this is f [or nil]
-		   (bascdr (and f (cdr fx))) ;this is (x) [maybe (x,y..), or nil]
+		   (f (and (not (atom fx)) (atom (caar fx)) (caar fx))) ; this is f [or nil]
+		   (bascdr (and f (cdr fx))) ; this is (x) [maybe (x,y..), or nil]
 		   (expon (caddr x)) ;; this is the exponent
-		   (doit (and 
+		   (doit (and
 			  f ; there is such a function
-			  (memq(getchar f 1) '(% $)) ;; insist it is a % or $ function
-			  (not (memq f '(%sum %product))) ;; what else? whata hack...
-			  (or (and(atom expon)(not(numberp expon))) ; f(x)^y is ok
-			      (and (atom expon)(numberp expon)(> expon 0))
-			      ; f(x)^3 is ok, but not f(x)^-1, which could 
+			  (memq (getchar f 1) '(% $)) ;; insist it is a % or $ function
+                          (not (eq (car (last (car fx))) 'array)) ; fix for x[i]^2
+                              ; Jesper Harder <harder@ifa.au.dk>
+                          (not (memq f '(%sum %product %derivative %integrate %at
+					      %lsum %limit))) ;; what else? what a hack...
+			  (or (and (atom expon) (not (numberp expon))) ; f(x)^y is ok
+			      (and (atom expon) (numberp expon) (> expon 0))))))
+			      ; f(x)^3 is ok, but not f(x)^-1, which could
 			      ; inverse of f, if written f^-1 x
-			      ); what else? f(x)^(1/2) is sqrt(f(x)), ??
-			  )))
-		  (cond(doit
+			      ; what else? f(x)^(1/2) is sqrt(f(x)), ??
+		  (cond (doit
 			(setq l (tex `((mexpt) ,f ,expon) l nil 'mparen 'mparen))
-			(setq r (tex
-				 (cond((cdr bascdr)
-				       
-				       `((mprogn),@bascdr));;handle f(x,y)^2 as f^2(x,y)
-				      (t(car bascdr))) ;; but f(x)^2 as f^2 x.
-				 nil r 'mtimes 'mtimes)))
-		       (t nil); won't doit. fall through
-		       ))))
-      (t(setq l (tex (cadr x) l nil lop (caar x))
-	      r (if (mmminusp (setq x (nformat (caddr x))))
+			(if (and (null (cdr bascdr))
+				 (eq (get f 'tex) 'tex-prefix))
+			    (setq r (tex (car bascdr) nil r f 'mparen))
+			  (setq r (tex (cons '(mprogn) bascdr) nil r 'mparen 'mparen))))
+		        (t nil))))) ; won't doit. fall through
+      (t (setq l (tex (cadr x) l nil lop (caar x))
+	       r (if (mmminusp (setq x (nformat (caddr x))))
 		    ;; the change in base-line makes parens unnecessary
 		    (if nc
 			(tex (cadr x) '("^ {-\\langle ")(cons "\\rangle }" r) 'mparen 'mparen)
 			(tex (cadr x) '("^ {- ")(cons " }" r) 'mparen 'mparen))
-		    
 		    (if nc
 			(tex x (list "^{\\langle ")(cons "\\rangle}" r) 'mparen 'mparen)
-			(tex x (list "^{")(cons "}" r) 'mparen 'mparen))))))
-
+			(if (and (numberp x) (< x 10))
+			    (tex x (list "^")(cons "" r) 'mparen 'mparen)
+			    (tex x (list "^{")(cons "}" r) 'mparen 'mparen))
+			)))))
       (append l r)))
 
 (defprop mncexpt tex-mexpt tex)
@@ -463,14 +528,11 @@
 (defprop mnctimes 110. tex-lbp)
 (defprop mnctimes 109. tex-rbp)
 
-(defprop mtimes tex-mtimes tex)
+(defprop mtimes tex-nary tex)
+(defprop mtimes "\\," texsym)
 (defprop mtimes 120. tex-lbp)
 (defprop mtimes 120. tex-rbp)
 
-(defun tex-mtimes (x l r) (let ((lop 'mtimes) ;; is this right?
-				(rop 'mtimes))
-			       (texnary x l r "\\>")))
-
 (defprop %sqrt tex-sqrt tex)
 
 (defun tex-sqrt(x l r)
@@ -487,7 +549,7 @@
 (defprop mquotient tex-mquotient tex)
 (defprop mquotient ("\\over") texsym)
 (defprop mquotient 122. tex-lbp) ;;dunno about this
-(defprop mquotient 123. tex-rbp) 
+(defprop mquotient 123. tex-rbp)
 
 (defun tex-mquotient (x l r)
   (if (or (null (cddr x)) (cdddr x)) (wna-err (caar x)))
@@ -501,7 +563,7 @@
 (defun tex-matrix(x l r) ;;matrix looks like ((mmatrix)((mlist) a b) ...)
   (append l `("\\pmatrix{")
 	 (mapcan #'(lambda(y)
-			  (tex-list (cdr y) nil (list "\\cr ") "&")) 
+			  (tex-list (cdr y) nil (list "\\cr ") "&"))
 		 (cdr x))
 	 '("}") r))
 
@@ -509,16 +571,27 @@
 ;; TeX is lots more flexible .. but
 
 (defprop %sum tex-sum tex)
+(defprop %lsum tex-lsum tex)
 (defprop %product tex-sum tex)
 
 ;; easily extended to union, intersect, otherops
 
+(defun tex-lsum(x l r)
+  (let ((op (cond ((eq (caar x) '%lsum) "\\sum_{")
+		  ;; extend here
+		  ))
+	;; gotta be one of those above 
+	(s1 (tex (cadr x) nil nil 'mparen rop));; summand
+	(index ;; "index = lowerlimit"
+	       (tex `((min simp) , (caddr x), (cadddr x))  nil nil 'mparen 'mparen)))
+       (append l `( ,op ,@index "}}{" ,@s1 "}") r)))
+
 (defun tex-sum(x l r)
   (let ((op (cond ((eq (caar x) '%sum) "\\sum_{")
 		  ((eq (caar x) '%product) "\\prod_{")
 		  ;; extend here
 		  ))
-	;; gotta be one of those above 
+	;; gotta be one of those above
 	(s1 (tex (cadr x) nil nil 'mparen rop));; summand
 	(index ;; "index = lowerlimit"
 	       (tex `((mequal simp) ,(caddr x),(cadddr x)) nil nil 'mparen 'mparen))
@@ -530,19 +603,19 @@
   (let ((s1 (tex (cadr x) nil nil 'mparen 'mparen));;integrand delims / & d
 	(var (tex (caddr x) nil nil 'mparen rop))) ;; variable
        (cond((= (length x) 3)
-	     (append l `("\\int {" ,@s1 "}{\\>d" ,@var "}") r))
+	     (append l `("\\int {" ,@s1 "}{\\;d" ,@var "}") r))
 	    (t ;; presumably length 5
 	       (let ((low (tex (nth 3 x) nil nil 'mparen 'mparen))
 		     ;; 1st item is 0
 		     (hi (tex (nth 4 x) nil nil 'mparen 'mparen)))
-		    (append l `("\\int_{" ,@low "}^{" ,@hi "}{" ,@s1 "\\>d" ,@var "}") r))))))
+		    (append l `("\\int_{" ,@low "}^{" ,@hi "}{" ,@s1 "\\;d" ,@var "}") r))))))
 
 (defprop %limit tex-limit tex)
 
 (defun tex-limit(x l r) ;; ignoring direction, last optional arg to limit
   (let ((s1 (tex (cadr x) nil nil 'mparen rop));; limitfunction
 	(subfun ;; the thing underneath "limit"
-	 (subst "\\to " '=
+	 (subst "\\rightarrow " '=
 		(tex `((mequal simp) ,(caddr x),(cadddr x))
 		     nil nil 'mparen 'mparen))))
        (append l `("\\lim_{" ,@subfun "}{" ,@s1 "}") r)))
@@ -553,30 +626,39 @@
 (defun tex-at (x l r)
   (let ((s1 (tex (cadr x) nil nil lop rop))
 	(sub (tex (caddr x) nil nil 'mparen 'mparen)))
-       (append l s1  '("\\bigg|_{") sub '("}") r)))
+       (append l '("\\left.") s1  '("\\right|_{") sub '("}") r)))
+
+(defprop mbox tex-mbox tex)
+
+;; \boxed is defined in amsmath.sty,
+;; \newcommand{\boxed}[1]{\fbox{\m@th$\displaystyle#1$}}
+
+(defun tex-mbox (x l r)
+  (append l '("\\boxed{") (tex (cadr x) nil nil 'mparen 'mparen) '("}") r))
+
+(defprop mlabox tex-mlabox tex)
+
+(defun tex-mlabox (x l r)
+   (append l '("\\stackrel{") (tex (caddr x) nil nil 'mparen 'mparen)
+	   '("}{\\boxed{") (tex (cadr x) nil nil 'mparen 'mparen) '("}}") r))
 
 ;;binomial coefficients
 
 (defprop %binomial tex-choose tex)
-	   
+
 (defun tex-choose (x l r)
-  `(,@l 
-    "{" 
+  `(,@l
+    "\\pmatrix{"
     ,@(tex (cadr x) nil nil 'mparen 'mparen)
-    "\\choose "
+    "\\\\"
     ,@(tex (caddr x) nil nil 'mparen 'mparen)
     "}"
     ,@r))
 
-
-(defprop rat tex-rat tex) 
+(defprop rat tex-rat tex)
 (defprop rat 120. tex-lbp)
 (defprop rat 121. tex-rbp)
-(defun tex-rat(x l r)
-  (let ((p (tex-mquotient x nil nil))
-	(q  (list "{" (cadr x) "//" (caddr x) "}"))) 
-       ;; do we ever need (1/2) instead of 1/2 ?
-       `(,@l  "\\mathchoice " ,@p ,@p ,@q ,@q ,@r )))
+(defun tex-rat(x l r) (tex-mquotient x l r))
 
 (defprop mplus tex-mplus tex)
 (defprop mplus 100. tex-lbp)
@@ -589,7 +671,7 @@
 	(if (null (cdr x))
 	    (tex-function x l r t)
 	    (tex (cadr x) (cons "+" l) r 'mplus rop)))
-       (t (setq l (tex (cadr x) l nil lop 'mplus) 
+       (t (setq l (tex (cadr x) l nil lop 'mplus)
 		x (cddr x))
 	  (do ((nl l)  (dissym))
 	      ((null (cdr x))
@@ -607,6 +689,10 @@
 (defprop mminus 100. tex-rbp)
 (defprop mminus 100. tex-lbp)
 
+(defprop min tex-infix tex)
+(defprop min ("\\in{") texsym)
+(defprop min 80. tex-lbp)
+(defprop min 80. tex-rbp)
 
 (defprop mequal tex-infix tex)
 (defprop mequal (=) texsym)
@@ -623,7 +709,7 @@
 (defprop mgreaterp 80. tex-rbp)
 
 (defprop mgeqp tex-infix tex)
-(defprop mgeqp ("\\ge") texsym)
+(defprop mgeqp ("\\geq ") texsym)
 (defprop mgeqp 80. tex-lbp)
 (defprop mgeqp 80. tex-rbp)
 
@@ -633,21 +719,21 @@
 (defprop mlessp 80. tex-rbp)
 
 (defprop mleqp tex-infix tex)
-(defprop mleqp ("\\le") texsym) ;; etc
+(defprop mleqp ("\\leq ") texsym)
 (defprop mleqp 80. tex-lbp)
 (defprop mleqp 80. tex-rbp)
 
 (defprop mnot tex-prefix tex)
-(defprop mnot ("\\lnot ") texsym)
+(defprop mnot ("\\not ") texsym)
 (defprop mnot 70. tex-rbp)
 
 (defprop mand tex-nary tex)
-(defprop mand ("\\land") texsym) ;; fill in the rest of the operators..
+(defprop mand ("\\and") texsym)
 (defprop mand 60. tex-lbp)
 (defprop mand 60. tex-rbp)
 
 (defprop mor tex-nary tex)
-(defprop mor ("\\lor") texsym)
+(defprop mor ("\\or") texsym)
 
 ;; make sin(x) display as sin x , but sin(x+y) as sin(x+y)
 ;; etc
@@ -658,16 +744,47 @@
       (setf (get a 'tex) 'tex-prefix)
       (setf (get a 'texword) b)  ;This means "sin" will always be roman
       (setf (get a 'texsym) (list b))
-      (setf (get a 'tex-rbp) 110)))
+      (setf (get a 'tex-rbp) 130)))
 
-(mapc #'tex-setup 
-  '( (%sin "\\sin ")
-     (%cos "\\cos ")
+;; JM 09/01 expand and re-order to follow table of "log-like" functions,
+;; see table in Lamport, 2nd edition, 1994, p. 44, table 3.9.
+;; I don't know if these are Latex-specific so you may have to define
+;; them if you use plain Tex.
+
+(mapc #'tex-setup
+  '(
      (%acos "\\arccos ")
+     (%asin "\\arcsin ")
+     (%atan "\\arctan ")
+     ; Latex's arg(x) is ... ?
+     (%cos "\\cos ")
+     (%cosh "\\cosh ")
+     (%cot "\\cot ")
+     (%coth "\\coth ")
+     (%csc "\\csc ")
+     ; Latex's "deg" is ... ?
+     (%determinant "\\det ")
+     (%dim "\\dim ")
+     (%exp "\\exp ")
+     (%gcd "\\gcd ")
+     ; Latex's "hom" is ... ?
+     (%inf "\\inf ") ; many will prefer "\\infty". Hmmm.
+     ; Latex's "ker" is ... ?
+     ; Latex's "lg" is ... ?
+     ; lim is handled by tex-limit.
+     ; Latex's "liminf" ... ?
+     ; Latex's "limsup" ... ?
      (%ln "\\ln ")
      (%log "\\log ")
-     (%atan "\\arctan ")
+     (%max "\\max ")
+     (%min "\\min ")
+     ; Latex's "Pr" ... ?
+     (%sec "\\sec ")
+     (%sin "\\sin ")
+     (%sinh "\\sinh ")
+     ; Latex's "sup" ... ?
      (%tan "\\tan ")
+     (%tanh "\\tanh ")
     ;; (%erf "{\\rm erf}") this would tend to set erf(x) as erf x. Unusual
      ;(%laplace "{\\cal L}")
      )) ;; etc
@@ -681,9 +798,11 @@
 (defprop mcond 25. tex-rbp)
 (defprop %derivative tex-derivative tex)
 (defun tex-derivative (x l r)
-  (tex (tex-d x '$|d|) l r lop rop ))
+  (tex (if $derivabbrev
+	   (tex-dabbrev x)
+	   (tex-d x '$|d|)) l r lop rop ))
 
-(defun tex-d(x dsym) ;dsym should be $d or "$d\\partial"
+(defun tex-d(x dsym) ;dsym should be $d or "$\\partial"
   ;; format the macsyma derivative form so it looks
   ;; sort of like a quotient times the deriva-dand.
   (let*
@@ -691,7 +810,7 @@
     (difflist (cddr x)) ;; list of derivs e.g. (x 1 y 2)
     (ords (odds difflist 0)) ;; e.g. (1 2)
     (vars (odds difflist 1)) ;; e.g. (x y)
-    (numer `((mexpt) $|d| ((mplus) ,@ords))) ; d^n numerator
+    (numer `((mexpt) ,dsym ((mplus) ,@ords))) ; d^n numerator
     (denom (cons '(mtimes)
 		 (mapcan #'(lambda(b e)
 				  `(,dsym ,(simplifya `((mexpt) ,b ,e) nil)))
@@ -700,29 +819,40 @@
      ((mquotient) ,(simplifya numer nil) ,denom)
      ,arg)))
 
-(defun odds(n c) 
+(defun tex-dabbrev (x)
+  ;; Format diff(f,x,1,y,1) so that it looks like
+  ;; f
+  ;;  x y
+  (let*
+      ((arg (cadr x)) ;; the function being differentiated
+       (difflist (cddr x)) ;; list of derivs e.g. (x 1 y 2)
+       (ords (odds difflist 0))	;; e.g. (1 2)
+       (vars (odds difflist 1))) ;; e.g. (x y)
+    (append
+     (if (symbolp arg)
+	 `((,arg array))
+	 `((mqapply array) ,arg))
+     (if (and (= (length vars) 1)
+	      (= (car ords) 1))
+	 vars
+	 `(((mtimes) ,@(mapcan #'(lambda (var ord)
+				   (make-list ord :initial-element var))
+			       vars ords)))))))
+
+(defun odds(n c)
   ;; if c=1, get the odd terms  (first, third...)
-  (cond ((null n) nil) 
+  (cond ((null n) nil)
 	((= c 1)(cons (car n)(odds (cdr n) 0)))
 	((= c 0)(odds (cdr n) 1))))
-    
-(defun tex-mcond (x l r &aux if)
-    (setq if (nreconc l '("{\\bf if\\>}")) 
-	  l (tex (cadr x) nil nil 'mcond 'mparen))
-    (cond ((eq '$false (fifth x))
-	   (setq x (tex (caddr x)
-			  '("{\\bf \\> then\\>}")
-			  r 'mcond rop))
-	   (append if l x))
-	  (t (setq r (tex (fifth x)
-			    '("{\\bf \\> else \\>}")
-			    r 'mcond rop)
-		   x (tex (caddr x)
-			  '("{\\bf \\> then\\>}")
-			    
-			    nil 'mcond 'mparen))
-	     (append  if l x r))))
 
+(defun tex-mcond (x l r)
+  (append l
+    (tex (cadr x) '("\\mathbf{if}\\;")
+      '("\\;\\mathbf{then}\\;") 'mparen 'mparen)
+    (if (eql (fifth x) '$false)
+      (tex (caddr x) nil r 'mcond rop)
+      (append (tex (caddr x) nil nil 'mparen 'mparen)
+        (tex (fifth x) '("\\;\\mathbf{else}\\;") r 'mcond rop)))))
 
 (defprop mdo tex-mdo tex)
 (defprop mdo 30. tex-lbp)
@@ -736,41 +866,98 @@
 ;; these aren't quite right
 
 (defun tex-mdo (x l r)
-  (texnary (cons '(mdo) (texmdo x)) l r "\\>"))
+  (tex-list (texmdo x) l r "\\;"))
 
 (defun tex-mdoin (x l r)
-  (texnary (cons '(mdo) (texmdoin x)) l r "\\>"))
+  (tex-list (texmdoin x) l r "\\;"))
 
 (defun texmdo (x)
-   (nconc (cond ((second x) `("{\\bf for}" ,(second x))))
+   (nconc (cond ((second x) `("\\mathbf{for}" ,(second x))))
 	 (cond ((equal 1 (third x)) nil)
-	       ((third x)  `("{\\bf from}" ,(third x))))
+	       ((third x)  `("\\mathbf{from}" ,(third x))))
 	 (cond ((equal 1 (fourth x)) nil)
-	       ((fourth x) `("{\\bf step}" ,(fourth x)))
-	       ((fifth x)  `("{\\bf next}" ,(fifth x))))
-	 (cond ((sixth x)  `("{\\bf thru}" ,(sixth x))))
+	       ((fourth x) `("\\mathbf{step}" ,(fourth x)))
+	       ((fifth x)  `("\\mathbf{next}" ,(fifth x))))
+	 (cond ((sixth x)  `("\\mathbf{thru}" ,(sixth x))))
 	 (cond ((null (seventh x)) nil)
 	       ((eq 'mnot (caar (seventh x)))
-		`("{\\bf while}" ,(cadr (seventh x))))
-	       (t `("{\\bf unless}" ,(seventh x))))
-	 `("{\\bf do}" ,(eighth x))))
+		`("\\mathbf{while}" ,(cadr (seventh x))))
+	       (t `("\\mathbf{unless}" ,(seventh x))))
+	 `("\\mathbf{do}" ,(eighth x))))
 
 (defun texmdoin (x)
-  (nconc `("{\\bf for}" ,(second x) $|in| ,(third x))
-	 (cond ((sixth x) `("{\\bf thru}" ,(sixth x))))
+  (nconc `("\\mathbf{for}" ,(second x) "\\mathbf{in}" ,(third x))
+	 (cond ((sixth x) `("\\mathbf{thru}" ,(sixth x))))
 	 (cond ((null (seventh x)) nil)
 	       ((eq 'mnot (caar (seventh x)))
-		`("{\\bf while}" ,(cadr (seventh x))))
-	       (t `("{\\bf unless}" ,(seventh x))))
-	 `("{\\bf do}" ,(eighth x))))
+		`("\\mathbf{while}" ,(cadr (seventh x))))
+	       (t `("\\mathbf{unless}" ,(seventh x))))
+	 `("\\mathbf{do}" ,(eighth x))))
+
+(defprop mtext tex-mtext tex)
+(defprop text-string tex-mtext tex)
+(defprop mlable tex-mlable tex)
+(defprop spaceout tex-spaceout tex)
+
+;; Additions by Marek Rychlik (rychlik@u.arizona.edu)
+;; This stuff handles setting of LET rules
+
+(defprop | --> | "\\longrightarrow " texsym)
+(defprop | WHERE | "\\;\\mathbf{where}\\;}" texsym)
+
+(defprop &>= ("\\ge ") texsym)
+(defprop &>= tex-infix tex)
+
+(defprop &> (">") texsym)
+(defprop &> tex-infix tex)
+
+(defprop &<= ("\\le ") texsym)
+(defprop &<= tex-infix tex)
+
+(defprop &< ("<") texsym)
+(defprop &< tex-infix tex)
+
+(defprop &= ("=") texsym)
+(defprop &= tex-infix tex)
+
+(defprop |&#| ("\\ne ") texsym)
+(defprop |&#| tex-infix tex)
+
+;; handles portrayal of rules, e.g.
+;; let(abs(x),x, ">" , x, 0);
+;; results in
+;; ((MTEXT SIMP) ((MABS SIMP) |$x|) | --> | |$x| | WHERE | ((&> SIMP) |$x| 0))
+
+(defun tex-mtext (x l r)
+  (tex-list (list* (cadr x)
+                 (texsym (caddr x))
+                 (cadddr x)
+                 (cond ((cddddr x)
+                        (list*
+                         (texsym (caddddr x))
+                         (cdddddr x)))))
+          l r ""))
+
+;; end of additions by Marek Rychlik
+
+(defun tex-mlable (x l r)
+  (tex (caddr x)
+    (append l
+      (if (cadr x)
+        (list (format nil "\\mbox{\\tt\\red(~A) \\black}" (tex-stripdollar (cadr x))))
+        nil))
+    r 'mparen 'mparen))
+
+(defun tex-spaceout (x l r)
+  (append l (list "\\mbox{\\verb|" (make-string (cadr x) :initial-element #\space) "|}") r))
 
 ;; initialize a file so that c-lines will look ok in verbatim mode
 ;; run this first before tex(<whatever>, file);
 (defun $texinit(file)
   ;; copy header from some generic place
-  (funcall 'exec (list 
+  (funcall 'exec (list
 		  (concat "cp "
-			  vaxima-main-dir 
+			  vaxima-main-dir
 			  "//ucb//verbwin "  ;extra slashes for maclisp // = /
 			  (stripdollar file))))
   '$done )
@@ -781,22 +968,61 @@
 (defun $texend(filename)
   (with-open-file (st      (stripdollar filename)
 				   :direction :output
-				   :if-exists :append)
+				   :if-exists :append
+				   :if-does-not-exist :create)
   (format st "\\end~%")
   '$done))
 
-;; Undone and easy:
-;; type in all the greek letters and other funny stuff that TeX
-;; knows about.
-
 ;; Undone and trickier:
 ;; handle reserved symbols stuff, just in case someone
-;; has a macsyma variable named (yuck!!) \over  or has a name with 
+;; has a macsyma variable named (yuck!!) \over  or has a name with
 ;; {} in it.
-;; Maybe do some special hacking for standard notations for 
+;; Maybe do some special hacking for standard notations for
 ;; hypergeometric fns, alternative summation notations  0<=n<=inf, etc.
 
 ;;Undone and really pretty hard: line breaking
 
-nil
+;;  The texput function was written by Barton Willis.
 
+(defun $texput (e s &optional tx)
+
+  (cond ((mstringp e)
+	 (setq e (define-symbol (string-left-trim '(#\&) e)))))
+
+  (cond (($listp s)
+	 (setq s (margs s)))
+	(t
+	 (setq s (list s))))
+  
+  (setq s (mapcar #'stripdollar s))
+
+  (cond ((null tx)
+	 (putprop e (nth 0 s) 'texword))
+	
+	((eq tx '$matchfix)
+	 (putprop e 'tex-matchfix 'tex)
+	 (cond ((< (length s) 2)
+		(merror "Improper 2nd argument to TEXPUT for matchfix operator."))
+	       ((eq (length s) 2)
+		(putprop e (list (list (nth 0 s)) (nth 1 s)) 'texsym))
+	       (t
+		(putprop e (list (list (nth 0 s)) (nth 1 s) (nth 2 s)) 'texsym))))
+
+          ; The left and right binding powers may be wrong.
+
+	((eq tx '$prefix)
+	 (putprop e 'tex-prefix 'tex)
+	 (putprop e s 'texsym)
+	 (putprop e 200 'tex-lbp)
+	 (putprop e 180 'tex-rbp))
+		
+	((eq tx '$infix)
+	 (putprop e 'tex-infix 'tex)
+	 (putprop e  s 'texsym)
+	 (putprop e 200 'tex-lbp)
+	 (putprop e 180 'tex-rbp))
+
+	((eq tx '$postfix)
+	 (putprop e 'tex-postfix 'tex)
+	 (putprop e  s 'texsym)
+	 (putprop e 160 'tex-lbp))))
