@@ -72,9 +72,26 @@
 (autof '$specint '|hypgeo|)
 (autof 'HGFSIMP-EXEC '|hyp|)
 (autof '$hgfred '|hyp|)
+;; autoload mexpr from a .lsp file
 (auto-mexpr '$trigrat '|trigrat|)
+;; autoload from a .mc file
+($auto_mexpr '$gcdex '|gcdex|)
 
+($auto_mexpr '$expandwrt "simplification/stopex")
+($auto_mexpr '$expandwrt_factored "simplification/stopex")
+(declaim (special $expandwrt_denom $expandwrt_nonrat))
 
+($auto_mexpr '$facsum "simplification/facexp")
+($auto_mexpr '$factorfacsum "simplification/facexp")
+($auto_mexpr '$collectterms "simplification/facexp")
+(declaim (special $nextlayerfactor $facsum_combine))
+
+($auto_mexpr '$disolate "simplification/disol")
+
+($auto_mexpr '$linsimp "misc/declin")
+($auto_mexpr '$declare_linear_operator "misc/declin")
+
+($auto_mexpr '$nonumfactor "simplification/genut")
 
 ;;From main source files.
 (AUTOF '$GET '|maxmac|)
@@ -1150,12 +1167,13 @@
 	 $TREILLIS
 	 $TREINAT
 	 ))
-  (setf (get v 'autoload)        "sym_auto.mac")
+  (setf (get v 'autoload)        "sym.mac")
   )
 
 
 ;; make some externals for the bothcases stuff...
 ;; just want the symbols 2 cased..
+'$parametric
 '(defvar $plot_options '((mlist)
 			;((mlist) $x -3 3)
 			;((mlist) $y -3 3)
