@@ -94,10 +94,11 @@
       ;; This error should not happen in compiled code because
       ;; this check is done at compile time too.
       (MAXIMA-ERROR 'WRNG-NO-ARGS 'MFORMAT))
-  (LET ((STREAM (ARG 1))
-	(sSTRING (exploden (check-out-of-core-string (ARG 2))))
-	(arg-index 2))
-    #+NIL
+  (LET* ((STREAM (ARG 1))
+	 (sSTRING (exploden (check-out-of-core-string (ARG 2))))
+	 (arg-index 2))
+	;(or (eql (car sstring) #\&) (push #\& sstring))
+	
     (AND (OR (NULL STREAM)
 	     (EQ T STREAM))
 	 (SETQ STREAM *standard-output*))
@@ -106,8 +107,7 @@
     ;; If the lack of flexibilty becomes an issue then
     ;; it can be changed easily.
     (MFORMAT-LOOP (OUTPUT-TEXT))
-    ;; On Multics keep from getting bitten by line buffering.
-    #+Multics
+    ;; Keep from getting bitten by buffering.
     (FORCE-OUTPUT STREAM)
     ))
 
