@@ -108,6 +108,53 @@
 ;; functions are stored in the function cell (expr,fexpr,lexpr), whereas
 ;; the macsyma functions (mfexpr*,...) are stored on the property list.
 ;;
+
+
+;; 1) The variety of maxima functions is much more restricted than
+;;    what the table above shows.  I think the following table gives
+;;    the correct picture (like its counterpart, it ignores maxima
+;;    macros or functional arrays).
+;;
+;;
+;; Maxima function	shadow type	hook type	mget
+;; ____________________________________________________________
+;;	expr		 expr		  expr
+;;	mexpr		 expr		  expr		t
+;;	mfexpr*		 mfexpr*	  expr
+;;
+;; These types have the following meaning: Suppose MFUN evaluates to some
+;; symbol in the MAXIMA package.  That this symbol is of type
+;;
+;;  - EXPR (or SUBR) implies that it has a lisp function definition
+;;    (SYMBOL-FUNCTION MFUN).
+;;
+;;  - MEXPR implies that it has a (parsed) maxima language definition
+;;    (MGET MFUN 'MEXPR) and all arguments are evaluated by MEVAL.
+;;
+;;  - MFEXPR* implies that it has a lisp function definition
+;;    (GET MFUN 'MFEXPR*) and its arguments are not automatically
+;;    evaluated by MEVAL.
+;;
+;; Note that the shadow type has to agree with the original function's
+;; type in the way arguments are evaluated.  On the other hand, I think
+;; we are left with EXPR as the only hook type; as a matter of fact, this
+;; is equivalent to the next point:
+;;
+;; 2) There is no need for MAKE-TRACE-HOOK to dispatch with respect to
+;; HOOK-TYPE since, roughly speaking, proper handling of the traced
+;; function's arguments is done by the trace handler in concert with
+;; MEVAL*.
+;;
+;; Note that I also removed the COPY-LIST used to pass the traced
+;; function's argument list to the trace handler.
+;;
+;; There remains an annoying problem with translated functions: tracing
+;; some function of type MEXPR and then loading its translated version
+;; (which is of type EXPR) will not cleanly untrace it (i.e., it is
+;; effectively no longer traced but it remains on the list of traced
+;; functions).  I think that this has to be fixed somewhere in the
+;; translation package. -wj
+
 
 ;;; Structures.
 
@@ -171,7 +218,8 @@
 (defun macsyma-trace (fun) (macsyma-trace-sub fun 'trace-handler $trace))
 
 (defun macsyma-trace-sub (fun handler ilist &aux temp)
-  (cond ((not (symbolp fun))
+  (declare (symbol temp))		; pathetic
+  (cond ((not (symbolp (setq fun (getopr fun))))
 	 (mtell "~%Bad arg to TRACE: ~M" fun)
 	 nil)
 	((trace-p fun)
@@ -184,17 +232,17 @@
 	  "~%The function ~:@M cannot be traced because: ASK GJC~%"
 	  fun)
 	 nil)
-	((null (setq temp (macsyma-fsymeval fun)))
+	((not (setq temp (car (macsyma-fsymeval fun))))
 	 (mtell "~%~@:M has no functional properties." fun)
 	 nil)
-	((memq (car temp) '(mmacro translated-mmacro))
+	((memq temp '(mmacro translated-mmacro))
 	 (mtell "~%~@:M is a macro, won't trace well, so use ~
 		     the MACROEXPAND function to debug it." fun)
 	 nil)
-	((get (car temp) 'shadow)
-	 (put-trace-info fun (car temp) ilist)
-	 (trace-fshadow fun (car temp)
-			(make-trace-hook fun (car temp) handler))
+	((get temp 'shadow)
+	 (put-trace-info fun temp ilist)
+	 (trace-fshadow fun temp
+			(make-trace-hook fun temp handler))
 	 (list fun))
 	(t
 	 (mtell "~%~@:M has functional properties not understood by TRACE"
@@ -207,7 +255,7 @@
 
 (defun macsyma-untrace-sub (fun handler ilist)
   (prog1
-   (cond ((not (symbolp fun))
+   (cond ((not (symbolp (setq fun (getopr fun))))
 	  (mtell "~%Bad arg to UNTRACE: ~M" fun)
 	  nil)
 	 ((not (trace-p fun))
@@ -260,8 +308,6 @@
 (defprop expr expr shadow)
 (defprop mfexpr*s mfexpr* shadow)
 (defprop mfexpr* mfexpr* shadow)
-(defprop fsubr fexpr shadow)
-(defprop fexpr fexpr shadow)
 
 #-Multics
 (progn
@@ -309,9 +355,6 @@
       (cond ((memq shadow '(expr subr))
 	     (setf (trace-oldfun fun) (and (fboundp fun) (symbol-function fun)))
 	     (setf (symbol-function  fun)  value))
-	    ((memq shadow '(fexpr fsubr))
-	     (setf (trace-oldfun fun) (symbol-function fun))
-	     (setf (symbol-function  fun)  (cons 'nlambda (cdr value))))
 	    (t (setplist fun
 			 `(,shadow ,value ,@(symbol-plist fun)))))))
 
@@ -331,19 +374,15 @@
 	    (putd fun nil))))
 
 #+cl
-(defun trace-unfshadow (fun type &aux new-fun)
-  (cond ((and (fboundp fun)
-	      (consp (setq new-fun (symbol-function fun)))
-	      (consp (second new-fun))
-	      (among 'trace-args (second new-fun)))
-  (cond ((memq type '(expr subr fexpr fsubr))
+(defun trace-unfshadow (fun type)
+  ;; At this point, we know that FUN is traced.
+  (cond ((memq type '(expr subr))
 	 (let ((oldf (trace-oldfun fun)))
 	   (if (not (null oldf))
 	     (setf (symbol-function  fun)  oldf)
 	     (fmakunbound fun))))
 	 (t (remprop fun (get! type 'shadow))
 	    (fmakunbound fun))))
-	(t  (format t "~%This function was no longer defined as an interpreted-trace"))))
 
 ;--- trace-fsymeval :: find original function
 ;  fun : a function which is being traced.  The original defintion may
@@ -356,11 +395,10 @@
 		(if (eq (get! type-of 'shadow) type-of)
 		    (mget (cdr (mgetl fun (list type-of))) type-of)
 		    (mget fun type-of)))
-	       #+(or Franz Cl)
-	       ((memq (get! type-of 'shadow) '(expr fexpr))
+	       ((eq (get! type-of 'shadow) 'expr)
 		(trace-oldfun fun))
 	       (t (if (eq (get! type-of 'shadow) type-of)
-		      (get (cdr (getl fun (list type-of))) type-of)
+		      (cadr (getl (cdr (getl fun `(,type-of))) `(,type-of)))
 		      (get fun type-of)))))
       (trace-fsymeval
 	 (merror "Macsyma BUG: Trace property for ~:@M went away without hook."
@@ -463,7 +501,7 @@
 
 (defun trace-option-p (function KEYWORD)
   (do ((options					
-	(LET ((OPTIONS (TRACE-OPTIONS FUNCTION)))
+	(LET ((OPTIONS (TRACE-OPTIONS (getop FUNCTION))))
 	  (COND ((NULL OPTIONS) NIL)
 		(($LISTP OPTIONS) (CDR OPTIONS))
 		(T
@@ -479,7 +517,7 @@
 	  ((eq (caar option) keyword)
 	   (let ((return-to-trace-handle $trace_safety))
 	     (return (mapply (cadr option) predicate-arglist
-			     "&A trace option predicate")))))))
+			     '|&A trace option predicate|)))))))
 			
 
 (defun trace-enter-print (fun lev largs &aux (mlargs `((mlist) ,@largs)))
@@ -538,10 +576,10 @@
 (defun ask-choicep (llist &rest header-message)
        (do ((j 0 (f1+ j))
 	    (dlist nil
-		   (list* "M" `((marrow) ,j ,(car ilist)) dlist))
+		   (list* #\newline `((marrow) ,j ,(car ilist)) dlist))
 	    (ilist llist (cdr ilist)))
 	   ((null ilist)
-	    (setq dlist (nconc header-message (cons "M" (nreverse dlist))))
+	    (setq dlist (nconc header-message (cons #\newline (nreverse dlist))))
 	    (let ((upper (f1- j)))
 		 (pred-$read #'(lambda (val)
 				      (and (integerp val)
@@ -559,7 +597,7 @@
 			     "Exit with user supplied value")
 			   "Error during application of" (mopstringnam fun)
 			   "at level" level
-			   "M" "Do you want to:")
+			   #\newline "Do you want to:")
 	      ((0)
 	       '(MAXIMA-ERROR))
 	      ((1)
@@ -598,7 +636,7 @@
 
        (let ((mprops (mgetl fun '(mexpr mmacro)))
 	     (lprops (getl  fun '(translated-mmacro mfexpr* mfexpr*s)))
-	     (fcell-props (getl-fun fun '(subr lsubr expr fexpr macro fsubr))))
+	     (fcell-props (getl-fun fun '(subr lsubr expr macro))))
 	    (cond ($TRANSRUN
 		   ;; the default, so its really a waste to have looked for
 		   ;; those mprops. Its better to fix the crock than to
@@ -611,8 +649,6 @@
 (DEFPROP MEXPR EXPR HOOK-TYPE)
 (Defprop SUBR EXPR HOOK-TYPE)
 (Defprop LSUBR EXPR HOOK-TYPE)
-(Defprop FEXPR FEXPR HOOK-TYPE)
-(Defprop FSUBR FEXPR HOOK-TYPE)
 (Defprop MFEXPR* MACRO HOOK-TYPE)
 (Defprop MFEXPR*S MACRO HOOK-TYPE)
 
@@ -644,17 +680,11 @@
 
 #+cl
 (defun make-trace-hook (fun type handler)
-  (CASE (GET! TYPE 'HOOK-TYPE)
-    ((EXPR)
-     `(lambda (&rest trace-args)
-	(,handler ',fun (copy-list trace-args))))
-    ((FEXPR)
-     `(LAMBDA (&quote &rest TRACE-ARGL)
-	(,HANDLER ',FUN (copy-list TRACE-ARGL))))
-;;;???
-    ((MACRO)
-     `(lambda (&quote &rest TRACE-FORM)
-	(,HANDLER (CAAR TRACE-FORM) (copy-list TRACE-FORM))))))
+  ;; Argument handling according to FUN's TYPE is already done
+  ;; elsewhere: HANDLER, meval...
+  (declare (ignore type))
+  #'(lambda (&rest trace-args)
+      (funcall handler fun trace-args)))
        
 #+Maclisp
 (defmacro trace-setup-call (prop fun type)  
@@ -675,7 +705,7 @@
 	     (return-to-trace-handle nil))
 	    (case type
 		   ((mexpr)
-		    (mapply prop largs "&A traced function"))
+		    (mapply prop largs '|&A traced function|))
 		   ((expr)
 		    (apply prop largs))
 		   ((subr lsubr)
@@ -688,11 +718,8 @@
 		   ((MFEXPR*)
 		    (FUNCALL PROP (CAR LARGS)))
 		   ((MFEXPR*S)
-		    (SUBRCALL NIL PROP (CAR LARGS)))
-		   ((FEXPR)
-		    (FUNCALL PROP LARGS))
-		   ((FSUBR)
-		    (SUBRCALL NIL PROP LARGS)))))
+		    (SUBRCALL NIL PROP (CAR LARGS))))))
+
 
 ;;; I/O cruft
 
@@ -737,7 +764,7 @@
 					    (CDR $TIMER)))))
 
 (DEFUN MICRO-TO-SEC (RUNTIME)
-  (MUL RUNTIME 1.0E-6 '$SEC))
+  (MUL RUNTIME (float (/ internal-time-units-per-second)) '$SEC))
 (DEFUN MICRO-PER-CALL-TO-SEC (RUNTIME CALLS)
   (DIV (MICRO-TO-SEC RUNTIME)
        (IF (ZEROP CALLS) 1 CALLS)))
