@@ -23,7 +23,8 @@
 ;;;
 ;;; First we create a copy of macros to help debug the beast
 
-(eval-when (:compile-toplevel :load-toplevel :execute)
+(eval-when #-gcl(:compile-toplevel :load-toplevel :execute) 
+	   #+gcl(load compile eval)
 (defpackage "NREGEX"
   (:use "COMMON-LISP")
   (:export
@@ -428,6 +429,7 @@
 	      `(lambda (string &key (start 0) (end (length string)))
 		 (declare (string string)
 			  (fixnum start end)
+		          (ignorable start)
 			  (optimize (speed 0) (compilation-speed 3)))
 		 (setf *regex-groupings* ,group)
 		 (block final-return
@@ -440,6 +442,7 @@
 	    `(lambda (string &key (start 0) (end (length string)))
 	       (declare (string string)
 			(fixnum start end)
+		        (ignorable start)
 			(optimize (speed 0) (compilation-speed 3)))
 	       (setf *regex-groupings* ,group)
 	       (block final-return
