@@ -34,7 +34,7 @@
 	     (DOWN (CADR ANS)))
 	    ((EQ '$DESCRIBE (CAAR ANS)) (MDESCRIBE (DECODE (CADR ANS))))
 	    (T (OPT-ERR)))
-      (SETQ ANS (RETRIEVE NIL NIL))))
+      (SETQ ANS (RETRIEVE ": " nil))))
 
 (DEFUN DOWN (NODE &AUX OPTS)
   (SETQ NODE (DECODE NODE) OPTS (DOWNS NODE))
