@@ -14,7 +14,7 @@
 ;;	THIS IS THE NEW RATIONAL FUNCTION PACKAGE PART 3.
 ;;	IT INCLUDES THE GCD ROUTINES AND THEIR SUPPORTING FUNCTIONS
 
-(DECLARE-TOP(GENPREFIX A_3))
+(DECLARE-TOP(GENPREFIX A_3))
 
 (LOAD-MACSYMA-MACROS RATMAC)
 
