@@ -23,7 +23,7 @@
 
 (LOAD-MACSYMA-MACROS RZMAC RATMAC)
 
-(DECLARE-TOP(GENPREFIX A_5))
+(DECLARE-TOP(GENPREFIX A_5))
 
 (DEFMVAR GENVAR NIL
 	 "List of gensyms used to point to kernels from within polynomials.
@@ -292,8 +292,8 @@
 		    (SETQ ALPHA (MEQHK (ARG 2)))
 		    (NEWVAR ALPHA)
 		    (SETQ MINPOLY* (CADR (RATREP* ALPHA)))
-		    (IF (OR (NOT (UNIVAR (CDR MINPOLY*)))
-			    (PCOEFP MINPOLY*)
+		    (IF (OR (PCOEFP MINPOLY*)
+			    (NOT (UNIVAR (CDR MINPOLY*)))
 			    (f< (CADR MINPOLY*) 2))
 			(MERROR
 			  "The second argument to FACTOR must be a non-linear, univariate polynomial:~%~M"
@@ -438,6 +438,12 @@ ON	(SETQ X (CONS (PGCD (CAR X) (CAR Y)) (PLCM (CDR X) (CDR Y))))
 	      (SETQ VARLIST (CDR (LISTIFY NARGS)))
 	      (NEWVAR X)
 	      (DESETQ (H X . Y) (RATREP* X))
+	      (unless (atom x)
+		;; (CAR X) => gensym corresponding to apparent main var.
+		;; MAIN-GENVAR => gensym corresponding to the genuine main var.
+		(let ((main-genvar (nth (1- (length varlist)) genvar)))
+		  (unless (eq (car x) main-genvar)
+		    (setq x `(,main-genvar 0 ,x)))))
 	      (SETQ X (RCONTENT X)
 		    Y (CONS 1 Y))
 	      (SETQ H (LIST '(MLIST)
