@@ -248,8 +248,8 @@
 	    0))
 	  ((= 2 n)
 	   (psp2form (m* (m+ 1 *index)
-			 (m^ c (m* -1 (m+ 2 *index)))
-			 (m^ (m* -1 a) *index))
+			 (m^ a (m* -1 (m+ 2 *index)))
+			 (m^ (m* -1 c) *index))
 		     (if (equal m 1) *index (m* *index m))
 		     0))
 	  (t (psp2form (m* (do ((nn (f1- n) (f1- nn))
