@@ -184,8 +184,7 @@
 ;; seems to be used very rarely, so it seems hardly worth the effort.  On the
 ;; Lisp Machine, this is probably more efficient as simply (EQUAL A 1).
 
-#+(and cl (not cmu))
-(defmacro onep (a) `(eql ,a 1))
+;; (defmacro onep (a) `(eql ,a 1))
 
 #+cl
 (DEFMFUN ONEP1 (A) (OR (and (numberp a) (= A 1)) (EQUAL A BIGFLOATONE)))
@@ -660,7 +659,8 @@
 ;  any more bits than can be represented in the significand of a floating
 ;  point number.
 (eval-when (compile load)
-(defvar machine-mantissa-precision 24 );;is this correct 
+(defconstant machine-mantissa-precision
+  (float-digits 1d0))
 )
 (DEFUN FPCOFRAT1 (NU D)
   (IF (AND (BIGP NU) (BIGP D))
