@@ -64,7 +64,8 @@
 		(ELEMXPT (LIST '(MEXPT)
 			       BASE*
 			       (SIMPLIFY (LIST '(MTIMES)
-					   (LIST '(%LOG)
+					       (list '(mexpt)
+						     (list '(%log) base*) -1)					   (LIST '(%LOG)
 						 (CADR EXP))
 					   (CADDR EXP))))))
 	       ((NOT (SETQ W
@@ -368,7 +369,10 @@
   (cond ((alike1 (car wrt) var)
 	 (if (equal (cadr wrt) 1)		;Power = 1?
 	     (if (null (cddr wrt))		;single or partial
-		 expr				;single
+		 (if (null old-wrt)
+		     expr                     ;single
+		   `((%derivative), expr    ;partial in old-wrt
+		     ,.(nreverse old-wrt)))
 		 `((%derivative) ,expr		;Partial, return rest
 		   ,.(nreverse old-wrt)
 		   ,@(cddr wrt)))
@@ -472,7 +476,14 @@
 	((MEMQ (CAAR EX) '(MPLUS MTIMES))
 	 (DO ((U (CDR EX) (CDR U))) ((NULL U) T)
 	     (IF (NOT (RAT10 (CAR U))) (RETURN NIL))))
-	(T (RAT10 (CAR (MARGS EX))))))
+	(T
+	 (let ((examine (margs ex)))
+	   (if (atom (first examine))
+	       (do* ((element examine (rest element))
+		     (result (rat10 (first examine))
+			     (and result (rat10 (first element)))))
+		   ((or (null result) (null element)) result))
+	     (rat10 (first examine)))))))
 
 (DEFUN LISTGCD (POWERLIST)
   (PROG (P)
@@ -524,6 +535,11 @@
 	((NOT (FLOATP X)) 1)
 	(T (CDR (MAXIMA-RATIONALIZE X)))))
 
+;; EXP = f(t,u) where f is some function with, say, VAR = t,
+;; u^k = RATROOT = e*(a*t+b)/(c*t+d), where the smallest possible k
+;; is calculated below.
+;; As always, W is an alist which associates to the coefficients
+;; a, b... (and to VAR) their values.
 (DEFUN RATROOT (EXP VAR RATROOT W) 
 	 (PROG (ROOTLIST K Y W1) 
 	       (COND ((SETQ Y (CHEBYF EXP VAR)) (RETURN Y)))
@@ -558,7 +574,7 @@
 						  ((MEXPT) VAR ((MPLUS) -1 K))))))
 				   ((MEXPT) ((MPLUS)
 					     ((MTIMES) C ((MEXPT) VAR K))
-					     ((MTIMES) -1 A))
+					     ((MTIMES) -1 A e))
 					    2)))))
 		      VAR))
 	       (RETURN (SUBSTINT (SIMPLIFY (LIST '(MEXPT)
@@ -569,6 +585,7 @@
 
 (DEFUN SUBST41 (EXP A B) (SUBST4 EXP)) 
 
+;; exp = a*t^r1*(c1+c2*t^q)^r2, where var = t.
 (DEFUN CHEBYF (EXP VAR) 
   (PROG (R1 R2 D1 D2 N1 N2 W Q) 
 	(COND ((NOT (SETQ W
@@ -585,6 +602,15 @@
 				((COEFFTT) (A FREEVAR)))
 			      NIL)))
 	       (RETURN NIL)))
+	(when (zerop1 (cdr (sassq 'c1 w #'nill)))
+	  (return
+	    (mul*
+	     ;; This factor is locally constant as long as t and
+	     ;; c2*t^q avoid log's branch cut.
+	     (subliss w '((mtimes) a ((mexpt) var ((mtimes) -1 q r2))
+			  ((mexpt) ((mtimes) c2 ((mexpt) var q)) r2)))
+	     (integrator
+	      (subliss w '((mexpt) var ((mplus) r1 ((mtimes) q r2)))) var))))
 	(SETQ Q (CDR (SASSQ 'Q W 'NILL)))
 	(SETQ 
 	 W
