@@ -132,27 +132,31 @@
 ;(array *S12* flonum 18.)
 
 (defvar *li2* (*array nil 'flonum 15.))
+(eval-when (load eval)
 (fillarray *li2* 
 	   '(14.0       1.93506430 .166073033 2.48793229e-2 4.68636196e-3
 	     1.0016275e-3 2.32002196e-4 5.68178227e-5 1.44963006e-5
 	     3.81632946e-6 1.02990426e-6 2.83575385e-7 7.9387055e-8
 	     2.2536705e-8 6.474338e-9))
+)
 
 (defvar *li3* (*array nil 'flonum 15.))
+(eval-when (load eval)
 (fillarray *li3*
 	   '(14.0       1.95841721 8.51881315e-2 8.55985222e-3 1.21177214e-3
 	     2.07227685e-4 3.99695869e-5 8.38064066e-6 1.86848945e-6
 	     4.36660867e-7 1.05917334e-7 2.6478920e-8 6.787e-9 
 	     1.776536e-9 4.73417e-10))
-
+)
 (defvar *s12* (*array nil  'flonum 18.))
+(eval-when (load eval)
 (fillarray *S12*
 	   '(17.0      1.90361778 .431311318 .100022507 2.44241560e-2
 	     6.22512464e-3 1.64078831e-3 4.44079203e-4 1.22774942e-4
 	     3.45398128e-5 9.85869565e-6 2.84856995e-6 8.31708473e-7
 	     2.45039499e-7 7.2764962e-8 2.1758023e-8 6.546158e-9
 	     1.980328e-9))
-
+)
 (defun chebyli2 (x)
        (*$ x (cheby-prime (//$ (1+$ (*$ x 4.0)) 3.0) *li2*)))
 
