@@ -220,12 +220,12 @@ warning given about not-built-in modes being taken for MACSYMA EXTEND types.")
   (COND ((or val-givenp (not (eq v val)))
 	 ; hack because macsyma PROG binds variable
 	 ; to itself. 
-	 (let ((CHECKER (ASSQ MODE '(($FLOAT . FLOATP)
+	 (let ((CHECKER (ASSQ MODE `(($FLOAT . FLOATP)
 				     ($FIXNUM . INTEGERP)
 				     ($NUMBER . NUMBERP)
 				     ($LIST . $LISTP)
-				     ($BOOLEAN . (LAMBDA (U)
-						   (MEMQ U '(T NIL)))))))
+				     ($BOOLEAN . ,#'(LAMBDA (U)
+ 						      (MEMQ U '(T NIL)))))))
 	       (nchecker (assq mode '(($float . $real)
 				      ($fixnum . $integer)
 				      ($complex . $complex))))
@@ -267,27 +267,6 @@ warning given about not-built-in modes being taken for MACSYMA EXTEND types.")
 
 ;;; 1/2 is not $RATIONAL. bad name. it means CRE form.
 
-(DEFUN IR (X)
-       (CASE X
-	      (($FLOAT $REAL $FLOATP $FLONUM $FLOATNUM) '$FLOAT)
-	      (($FIXP $FIXNUM) '$FIXNUM)
-	      (($RATIONAL $RAT) '$RATIONAL)
-	      (($NUMBER $BIGNUM $BIG) '$NUMBER)
-	      (($BOOLEAN $BOOL) '$BOOLEAN)
-	      (($LIST $LISTP) '$LIST)
-	      (($ANY $NONE $ANY_CHECK) '$ANY)
-	      (T (UDM-ERR X) X)))
-
-(DEFUN UDM-ERR (MODE)
-       (MTELL "Warning:  ~:M is not a known mode declaration ~
-	      maybe you want ~:M mode.~%"
-	      MODE
-	      (CASE MODE
-		     (($INTEGER $INTEGERP) '$FIXNUM)
-		     (($COMPLEX) "&to ask about this")
-		     (($FUCKED $SHITTY) "&to watch your language")
-		     (T "&to see the documentation on"))))
-
 (DEFUN IR (X)
   (CASE X
 	 (($FLOAT $REAL $FLOATP $FLONUM $FLOATNUM) '$FLOAT)
